package com.mysql.jdbc;

import com.mysql.jdbc.profiler.ProfileEventSink;
import com.mysql.jdbc.profiler.ProfilerEvent;
import com.mysql.jdbc.util.ReadAheadInputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.SoftReference;
import java.math.BigInteger;
import java.net.Socket;
import java.security.NoSuchAlgorithmException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.zip.Deflater;

/* JADX INFO: Access modifiers changed from: package-private */
/* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/MysqlIO.class */
public class MysqlIO {
    private static final String CODE_PAGE_1252 = "Cp1252";
    protected static final int NULL_LENGTH = -1;
    protected static final int COMP_HEADER_LENGTH = 3;
    protected static final int MIN_COMPRESS_LEN = 50;
    protected static final int HEADER_LENGTH = 4;
    protected static final int AUTH_411_OVERHEAD = 33;
    private static int maxBufferSize;
    private static final int CLIENT_COMPRESS = 32;
    protected static final int CLIENT_CONNECT_WITH_DB = 8;
    private static final int CLIENT_FOUND_ROWS = 2;
    private static final int CLIENT_LOCAL_FILES = 128;
    private static final int CLIENT_LONG_FLAG = 4;
    private static final int CLIENT_LONG_PASSWORD = 1;
    private static final int CLIENT_PROTOCOL_41 = 512;
    private static final int CLIENT_INTERACTIVE = 1024;
    protected static final int CLIENT_SSL = 2048;
    private static final int CLIENT_TRANSACTIONS = 8192;
    protected static final int CLIENT_RESERVED = 16384;
    protected static final int CLIENT_SECURE_CONNECTION = 32768;
    private static final int CLIENT_MULTI_QUERIES = 65536;
    private static final int CLIENT_MULTI_RESULTS = 131072;
    private static final int SERVER_STATUS_IN_TRANS = 1;
    private static final int SERVER_STATUS_AUTOCOMMIT = 2;
    private static final int SERVER_MORE_RESULTS_EXISTS = 8;
    private static final int SERVER_QUERY_NO_GOOD_INDEX_USED = 16;
    private static final int SERVER_QUERY_NO_INDEX_USED = 32;
    private static final int SERVER_STATUS_CURSOR_EXISTS = 64;
    private static final String FALSE_SCRAMBLE = "xxxxxxxx";
    protected static final int MAX_QUERY_SIZE_TO_LOG = 1024;
    protected static final int MAX_QUERY_SIZE_TO_EXPLAIN = 1048576;
    protected static final int INITIAL_PACKET_SIZE = 1024;
    private static String jvmPlatformCharset;
    protected static final String ZERO_DATE_VALUE_MARKER = "0000-00-00";
    protected static final String ZERO_DATETIME_VALUE_MARKER = "0000-00-00 00:00:00";
    private static final int MAX_PACKET_DUMP_LENGTH = 1024;
    protected int serverCharsetIndex;
    private Buffer reusablePacket;
    private Buffer sendPacket;
    protected BufferedOutputStream mysqlOutput;
    protected ConnectionImpl connection;
    protected InputStream mysqlInput;
    private LinkedList packetDebugRingBuffer;
    protected Socket mysqlConnection;
    private SocketFactory socketFactory;
    private SoftReference loadFileBufRef;
    private SoftReference splitBufRef;
    protected String host;
    protected String seed;
    private String socketFactoryClassName;
    private boolean isInteractiveClient;
    private boolean logSlowQueries;
    private boolean profileSql;
    protected int port;
    protected int serverCapabilities;
    private Calendar sessionCalendar;
    private boolean useConnectWithDb;
    private boolean needToGrabQueryFromPacket;
    private boolean autoGenerateTestcaseScript;
    private long threadId;
    private boolean useNanosForElapsedTime;
    private long slowQueryThreshold;
    private String queryTimingUnits;
    private List statementInterceptors;
    private boolean useDirectRowUnpack;
    private int useBufferRowSizeThreshold;
    private boolean packetSequenceReset = false;
    private Buffer sharedSendPacket = null;
    private Deflater deflater = null;
    private RowData streamingData = null;
    private String serverVersion = null;
    private byte[] packetHeaderBuf = new byte[4];
    private boolean colDecimalNeedsBump = false;
    private boolean hadWarnings = false;
    private boolean has41NewNewProt = false;
    private boolean hasLongColumnInfo = false;
    private boolean platformDbCharsetMatches = true;
    private boolean queryBadIndexUsed = false;
    private boolean queryNoIndexUsed = false;
    private boolean use41Extensions = false;
    private boolean useCompression = false;
    private boolean useNewLargePackets = false;
    private boolean useNewUpdateCounts = false;
    private byte packetSequence = 0;
    private byte readPacketSequence = -1;
    private boolean checkPacketSequence = false;
    private byte protocolVersion = 0;
    private int maxAllowedPacket = MAX_QUERY_SIZE_TO_EXPLAIN;
    protected int maxThreeBytes = 16581375;
    private int serverMajorVersion = 0;
    private int serverMinorVersion = 0;
    private int serverStatus = 0;
    private int serverSubMinorVersion = 0;
    private int warningCount = 0;
    protected long clientParam = 0;
    protected long lastPacketSentTimeMs = 0;
    private boolean traceProtocol = false;
    private boolean enablePacketDebug = false;
    private int statementExecutionDepth = 0;

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:11:0x003a
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    static {
        /*
            r0 = 65535(0xffff, float:9.1834E-41)
            com.mysql.jdbc.MysqlIO.maxBufferSize = r0
            r0 = 0
            com.mysql.jdbc.MysqlIO.jvmPlatformCharset = r0
            r0 = 0
            r5 = r0
            java.io.OutputStreamWriter r0 = new java.io.OutputStreamWriter     // Catch: java.lang.Throwable -> L28
            r1 = r0
            java.io.ByteArrayOutputStream r2 = new java.io.ByteArrayOutputStream     // Catch: java.lang.Throwable -> L28
            r3 = r2
            r3.<init>()     // Catch: java.lang.Throwable -> L28
            r1.<init>(r2)     // Catch: java.lang.Throwable -> L28
            r5 = r0
            r0 = r5
            java.lang.String r0 = r0.getEncoding()     // Catch: java.lang.Throwable -> L28
            com.mysql.jdbc.MysqlIO.jvmPlatformCharset = r0     // Catch: java.lang.Throwable -> L28
            r0 = jsr -> L2e
        L25:
            goto L3d
        L28:
            r6 = move-exception
            r0 = jsr -> L2e
        L2c:
            r1 = r6
            throw r1
        L2e:
            r7 = r0
            r0 = r5
            if (r0 == 0) goto L37
            r0 = r5
            r0.close()     // Catch: java.io.IOException -> L3a
        L37:
            goto L3b
        L3a:
            r8 = move-exception
        L3b:
            ret r7
        L3d:
            return
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.MysqlIO.m11clinit():void");
    }

    public MysqlIO(String host, int port, Properties props, String socketFactoryClassName, ConnectionImpl conn, int socketTimeout, int useBufferRowSizeThreshold) throws IOException, SQLException {
        this.reusablePacket = null;
        this.sendPacket = null;
        this.mysqlOutput = null;
        this.mysqlInput = null;
        this.packetDebugRingBuffer = null;
        this.mysqlConnection = null;
        this.socketFactory = null;
        this.host = null;
        this.socketFactoryClassName = null;
        this.isInteractiveClient = false;
        this.logSlowQueries = false;
        this.profileSql = false;
        this.port = 3306;
        this.useDirectRowUnpack = true;
        this.connection = conn;
        if (this.connection.getEnablePacketDebug()) {
            this.packetDebugRingBuffer = new LinkedList();
        }
        this.useBufferRowSizeThreshold = useBufferRowSizeThreshold;
        this.useDirectRowUnpack = this.connection.getUseDirectRowUnpack();
        this.logSlowQueries = this.connection.getLogSlowQueries();
        this.reusablePacket = new Buffer(MysqlErrorNumbers.ER_ERROR_ON_READ);
        this.sendPacket = new Buffer(MysqlErrorNumbers.ER_ERROR_ON_READ);
        this.port = port;
        this.host = host;
        this.socketFactoryClassName = socketFactoryClassName;
        this.socketFactory = createSocketFactory();
        this.mysqlConnection = this.socketFactory.connect(this.host, this.port, props);
        if (socketTimeout != 0) {
            try {
                this.mysqlConnection.setSoTimeout(socketTimeout);
            } catch (Exception e) {
            }
        }
        this.mysqlConnection = this.socketFactory.beforeHandshake();
        if (this.connection.getUseReadAheadInput()) {
            this.mysqlInput = new ReadAheadInputStream(this.mysqlConnection.getInputStream(), CLIENT_RESERVED, this.connection.getTraceProtocol(), this.connection.getLog());
        } else if (this.connection.useUnbufferedInput()) {
            this.mysqlInput = this.mysqlConnection.getInputStream();
        } else {
            this.mysqlInput = new BufferedInputStream(this.mysqlConnection.getInputStream(), CLIENT_RESERVED);
        }
        this.mysqlOutput = new BufferedOutputStream(this.mysqlConnection.getOutputStream(), CLIENT_RESERVED);
        this.isInteractiveClient = this.connection.getInteractiveClient();
        this.profileSql = this.connection.getProfileSql();
        this.sessionCalendar = Calendar.getInstance();
        this.autoGenerateTestcaseScript = this.connection.getAutoGenerateTestcaseScript();
        this.needToGrabQueryFromPacket = this.profileSql || this.logSlowQueries || this.autoGenerateTestcaseScript;
        if (this.connection.getUseNanosForElapsedTime() && Util.nanoTimeAvailable()) {
            this.useNanosForElapsedTime = true;
            this.queryTimingUnits = Messages.getString("Nanoseconds");
        } else {
            this.queryTimingUnits = Messages.getString("Milliseconds");
        }
        if (this.connection.getLogSlowQueries()) {
            calculateSlowQueryThreshold();
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void initializeStatementInterceptors(String interceptorClasses, Properties props) throws SQLException {
        List interceptorsToCreate = StringUtils.split(interceptorClasses, ",", true);
        this.statementInterceptors = new LinkedList();
        Iterator iter = interceptorsToCreate.iterator();
        String classname = null;
        while (iter.hasNext()) {
            try {
                classname = iter.next().toString();
                StatementInterceptor interceptorInstance = (StatementInterceptor) Class.forName(classname).newInstance();
                interceptorInstance.init(this.connection, props);
                this.statementInterceptors.add(interceptorInstance);
            } catch (Throwable t) {
                SQLException sqlEx = SQLError.createSQLException(Messages.getString("MysqlIo.BadStatementInterceptor", new Object[]{classname}));
                sqlEx.initCause(t);
                throw sqlEx;
            }
        }
    }

    public boolean hasLongColumnInfo() {
        return this.hasLongColumnInfo;
    }

    protected boolean isDataAvailable() throws SQLException {
        try {
            return this.mysqlInput.available() > 0;
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public long getLastPacketSentTimeMs() {
        return this.lastPacketSentTimeMs;
    }

    protected ResultSetImpl getResultSet(StatementImpl callingStatement, long columnCount, int maxRows, int resultSetType, int resultSetConcurrency, boolean streamResults, String catalog, boolean isBinaryEncoded, Field[] metadataFromCache) throws SQLException {
        RowData rowData;
        Field[] fields = null;
        if (metadataFromCache == null) {
            fields = new Field[(int) columnCount];
            for (int i = 0; i < columnCount; i++) {
                Buffer fieldPacket = readPacket();
                fields[i] = unpackField(fieldPacket, false);
            }
        } else {
            for (int i2 = 0; i2 < columnCount; i2++) {
                skipPacket();
            }
        }
        Buffer packet = reuseAndReadPacket(this.reusablePacket);
        readServerStatusForResultSets(packet);
        if (this.connection.versionMeetsMinimum(5, 0, 2) && this.connection.getUseCursorFetch() && isBinaryEncoded && callingStatement != null && callingStatement.getFetchSize() != 0 && callingStatement.getResultSetType() == 1003) {
            ServerPreparedStatement prepStmt = (ServerPreparedStatement) callingStatement;
            boolean usingCursor = true;
            if (this.connection.versionMeetsMinimum(5, 0, 5)) {
                usingCursor = (this.serverStatus & SERVER_STATUS_CURSOR_EXISTS) != 0;
            }
            if (usingCursor) {
                RowData rows = new RowDataCursor(this, prepStmt, fields);
                ResultSetImpl rs = buildResultSetWithRows(callingStatement, catalog, fields, rows, resultSetType, resultSetConcurrency, isBinaryEncoded);
                if (usingCursor) {
                    rs.setFetchSize(callingStatement.getFetchSize());
                }
                return rs;
            }
        }
        if (!streamResults) {
            rowData = readSingleRowSet(columnCount, maxRows, resultSetConcurrency, isBinaryEncoded, metadataFromCache == null ? fields : metadataFromCache);
        } else {
            rowData = new RowDataDynamic(this, (int) columnCount, metadataFromCache == null ? fields : metadataFromCache, isBinaryEncoded);
            this.streamingData = rowData;
        }
        return buildResultSetWithRows(callingStatement, catalog, metadataFromCache == null ? fields : metadataFromCache, rowData, resultSetType, resultSetConcurrency, isBinaryEncoded);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public final void forceClose() {
        try {
            if (this.mysqlInput != null) {
                this.mysqlInput.close();
            }
        } catch (IOException e) {
            this.mysqlInput = null;
        }
        try {
            if (this.mysqlOutput != null) {
                this.mysqlOutput.close();
            }
        } catch (IOException e2) {
            this.mysqlOutput = null;
        }
        try {
            if (this.mysqlConnection != null) {
                this.mysqlConnection.close();
            }
        } catch (IOException e3) {
            this.mysqlConnection = null;
        }
    }

    protected final void skipPacket() throws SQLException {
        try {
            int lengthRead = readFully(this.mysqlInput, this.packetHeaderBuf, 0, 4);
            if (lengthRead < 4) {
                forceClose();
                throw new IOException(Messages.getString("MysqlIO.1"));
            }
            int packetLength = (this.packetHeaderBuf[0] & 255) + ((this.packetHeaderBuf[1] & 255) << 8) + ((this.packetHeaderBuf[2] & 255) << SERVER_QUERY_NO_GOOD_INDEX_USED);
            if (this.traceProtocol) {
                StringBuffer traceMessageBuf = new StringBuffer();
                traceMessageBuf.append(Messages.getString("MysqlIO.2"));
                traceMessageBuf.append(packetLength);
                traceMessageBuf.append(Messages.getString("MysqlIO.3"));
                traceMessageBuf.append(StringUtils.dumpAsHex(this.packetHeaderBuf, 4));
                this.connection.getLog().logTrace(traceMessageBuf.toString());
            }
            byte multiPacketSeq = this.packetHeaderBuf[COMP_HEADER_LENGTH];
            if (!this.packetSequenceReset) {
                if (this.enablePacketDebug && this.checkPacketSequence) {
                    checkPacketSequencing(multiPacketSeq);
                }
            } else {
                this.packetSequenceReset = false;
            }
            this.readPacketSequence = multiPacketSeq;
            skipFully(this.mysqlInput, packetLength);
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        } catch (OutOfMemoryError oom) {
            this.connection.realClose(false, false, true, oom);
            throw oom;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public final Buffer readPacket() throws SQLException {
        try {
            int lengthRead = readFully(this.mysqlInput, this.packetHeaderBuf, 0, 4);
            if (lengthRead < 4) {
                forceClose();
                throw new IOException(Messages.getString("MysqlIO.1"));
            }
            int packetLength = (this.packetHeaderBuf[0] & 255) + ((this.packetHeaderBuf[1] & 255) << 8) + ((this.packetHeaderBuf[2] & 255) << SERVER_QUERY_NO_GOOD_INDEX_USED);
            if (this.traceProtocol) {
                StringBuffer traceMessageBuf = new StringBuffer();
                traceMessageBuf.append(Messages.getString("MysqlIO.2"));
                traceMessageBuf.append(packetLength);
                traceMessageBuf.append(Messages.getString("MysqlIO.3"));
                traceMessageBuf.append(StringUtils.dumpAsHex(this.packetHeaderBuf, 4));
                this.connection.getLog().logTrace(traceMessageBuf.toString());
            }
            byte multiPacketSeq = this.packetHeaderBuf[COMP_HEADER_LENGTH];
            if (!this.packetSequenceReset) {
                if (this.enablePacketDebug && this.checkPacketSequence) {
                    checkPacketSequencing(multiPacketSeq);
                }
            } else {
                this.packetSequenceReset = false;
            }
            this.readPacketSequence = multiPacketSeq;
            byte[] buffer = new byte[packetLength + 1];
            int numBytesRead = readFully(this.mysqlInput, buffer, 0, packetLength);
            if (numBytesRead != packetLength) {
                throw new IOException(new StringBuffer().append("Short read, expected ").append(packetLength).append(" bytes, only read ").append(numBytesRead).toString());
            }
            buffer[packetLength] = 0;
            Buffer packet = new Buffer(buffer);
            packet.setBufLength(packetLength + 1);
            if (this.traceProtocol) {
                StringBuffer traceMessageBuf2 = new StringBuffer();
                traceMessageBuf2.append(Messages.getString("MysqlIO.4"));
                traceMessageBuf2.append(getPacketDumpToLog(packet, packetLength));
                this.connection.getLog().logTrace(traceMessageBuf2.toString());
            }
            if (this.enablePacketDebug) {
                enqueuePacketForDebugging(false, false, 0, this.packetHeaderBuf, packet);
            }
            return packet;
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        } catch (OutOfMemoryError oom) {
            this.connection.realClose(false, false, true, oom);
            throw oom;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public final Field unpackField(Buffer packet, boolean extractDefaultValues) throws SQLException {
        short colFlag;
        long colLength;
        short colFlag2;
        if (this.use41Extensions) {
            if (this.has41NewNewProt) {
                int catalogNameStart = packet.getPosition() + 1;
                int catalogNameLength = packet.fastSkipLenString();
                adjustStartForFieldLength(catalogNameStart, catalogNameLength);
            }
            int databaseNameStart = packet.getPosition() + 1;
            int databaseNameLength = packet.fastSkipLenString();
            int databaseNameStart2 = adjustStartForFieldLength(databaseNameStart, databaseNameLength);
            int tableNameStart = packet.getPosition() + 1;
            int tableNameLength = packet.fastSkipLenString();
            int tableNameStart2 = adjustStartForFieldLength(tableNameStart, tableNameLength);
            int originalTableNameStart = packet.getPosition() + 1;
            int originalTableNameLength = packet.fastSkipLenString();
            int originalTableNameStart2 = adjustStartForFieldLength(originalTableNameStart, originalTableNameLength);
            int nameStart = packet.getPosition() + 1;
            int nameLength = packet.fastSkipLenString();
            int nameStart2 = adjustStartForFieldLength(nameStart, nameLength);
            int originalColumnNameStart = packet.getPosition() + 1;
            int originalColumnNameLength = packet.fastSkipLenString();
            int originalColumnNameStart2 = adjustStartForFieldLength(originalColumnNameStart, originalColumnNameLength);
            packet.readByte();
            short charSetNumber = (short) packet.readInt();
            if (this.has41NewNewProt) {
                colLength = packet.readLong();
            } else {
                colLength = packet.readLongInt();
            }
            int colType = packet.readByte() & 255;
            if (this.hasLongColumnInfo) {
                colFlag2 = (short) packet.readInt();
            } else {
                colFlag2 = (short) (packet.readByte() & 255);
            }
            int colDecimals = packet.readByte() & 255;
            int defaultValueStart = -1;
            int defaultValueLength = -1;
            if (extractDefaultValues) {
                defaultValueStart = packet.getPosition() + 1;
                defaultValueLength = packet.fastSkipLenString();
            }
            Field field = new Field(this.connection, packet.getByteBuffer(), databaseNameStart2, databaseNameLength, tableNameStart2, tableNameLength, originalTableNameStart2, originalTableNameLength, nameStart2, nameLength, originalColumnNameStart2, originalColumnNameLength, colLength, colType, colFlag2, colDecimals, defaultValueStart, defaultValueLength, charSetNumber);
            return field;
        }
        int tableNameStart3 = packet.getPosition() + 1;
        int tableNameLength2 = packet.fastSkipLenString();
        int tableNameStart4 = adjustStartForFieldLength(tableNameStart3, tableNameLength2);
        int nameStart3 = packet.getPosition() + 1;
        int nameLength2 = packet.fastSkipLenString();
        int nameStart4 = adjustStartForFieldLength(nameStart3, nameLength2);
        int colLength2 = packet.readnBytes();
        int colType2 = packet.readnBytes();
        packet.readByte();
        if (this.hasLongColumnInfo) {
            colFlag = (short) packet.readInt();
        } else {
            colFlag = (short) (packet.readByte() & 255);
        }
        int colDecimals2 = packet.readByte() & 255;
        if (this.colDecimalNeedsBump) {
            colDecimals2++;
        }
        Field field2 = new Field(this.connection, packet.getByteBuffer(), nameStart4, nameLength2, tableNameStart4, tableNameLength2, colLength2, colType2, colFlag, colDecimals2);
        return field2;
    }

    private int adjustStartForFieldLength(int nameStart, int nameLength) {
        if (nameLength < 251) {
            return nameStart;
        }
        if (nameLength >= 251 && nameLength < CLIENT_MULTI_QUERIES) {
            return nameStart + 2;
        }
        if (nameLength >= CLIENT_MULTI_QUERIES && nameLength < 16777216) {
            return nameStart + COMP_HEADER_LENGTH;
        }
        return nameStart + 8;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean isSetNeededForAutoCommitMode(boolean autoCommitFlag) {
        if (this.use41Extensions && this.connection.getElideSetAutoCommits()) {
            boolean autoCommitModeOnServer = (this.serverStatus & 2) != 0;
            if (autoCommitFlag || !versionMeetsMinimum(5, 0, 0)) {
                return autoCommitModeOnServer != autoCommitFlag;
            }
            boolean inTransactionOnServer = (this.serverStatus & 1) != 0;
            return !inTransactionOnServer;
        }
        return true;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean inTransactionOnServer() {
        return (this.serverStatus & 1) != 0;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void changeUser(String userName, String password, String database) throws SQLException {
        this.packetSequence = (byte) -1;
        int userLength = userName != null ? userName.length() : 0;
        int databaseLength = database != null ? database.length() : 0;
        int packLength = ((userLength + SERVER_QUERY_NO_GOOD_INDEX_USED + databaseLength) * 2) + 7 + 4 + AUTH_411_OVERHEAD;
        if ((this.serverCapabilities & CLIENT_SECURE_CONNECTION) != 0) {
            Buffer changeUserPacket = new Buffer(packLength + 1);
            changeUserPacket.writeByte((byte) 17);
            if (versionMeetsMinimum(4, 1, 1)) {
                secureAuth411(changeUserPacket, packLength, userName, password, database, false);
                return;
            } else {
                secureAuth(changeUserPacket, packLength, userName, password, database, false);
                return;
            }
        }
        Buffer packet = new Buffer(packLength);
        packet.writeByte((byte) 17);
        packet.writeString(userName);
        if (this.protocolVersion > 9) {
            packet.writeString(Util.newCrypt(password, this.seed));
        } else {
            packet.writeString(Util.oldCrypt(password, this.seed));
        }
        boolean localUseConnectWithDb = this.useConnectWithDb && database != null && database.length() > 0;
        if (localUseConnectWithDb) {
            packet.writeString(database);
        }
        send(packet, packet.getPosition());
        checkErrorPacket();
        if (!localUseConnectWithDb) {
            changeDatabaseTo(database);
        }
    }

    protected Buffer checkErrorPacket() throws SQLException {
        return checkErrorPacket(-1);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void checkForCharsetMismatch() {
        if (this.connection.getUseUnicode() && this.connection.getEncoding() != null) {
            String encodingToCheck = jvmPlatformCharset;
            if (encodingToCheck == null) {
                encodingToCheck = System.getProperty("file.encoding");
            }
            if (encodingToCheck == null) {
                this.platformDbCharsetMatches = false;
            } else {
                this.platformDbCharsetMatches = encodingToCheck.equals(this.connection.getEncoding());
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void clearInputStream() throws SQLException {
        try {
            int len = this.mysqlInput.available();
            while (len > 0) {
                this.mysqlInput.skip(len);
                len = this.mysqlInput.available();
            }
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        }
    }

    protected void resetReadPacketSequence() {
        this.readPacketSequence = (byte) 0;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void dumpPacketRingBuffer() throws SQLException {
        if (this.packetDebugRingBuffer != null && this.connection.getEnablePacketDebug()) {
            StringBuffer dumpBuffer = new StringBuffer();
            dumpBuffer.append(new StringBuffer().append("Last ").append(this.packetDebugRingBuffer.size()).append(" packets received from server, from oldest->newest:\n").toString());
            dumpBuffer.append("\n");
            Iterator ringBufIter = this.packetDebugRingBuffer.iterator();
            while (ringBufIter.hasNext()) {
                dumpBuffer.append((StringBuffer) ringBufIter.next());
                dumpBuffer.append("\n");
            }
            this.connection.getLog().logTrace(dumpBuffer.toString());
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* JADX WARN: Code restructure failed: missing block: B:18:0x007e, code lost:
    
        if (r9 == null) goto L17;
     */
    /* JADX WARN: Code restructure failed: missing block: B:19:0x0081, code lost:
    
        r9.close();
     */
    /* JADX WARN: Code restructure failed: missing block: B:21:0x0089, code lost:
    
        if (r8 == null) goto L20;
     */
    /* JADX WARN: Code restructure failed: missing block: B:22:0x008c, code lost:
    
        r8.close();
     */
    /* JADX WARN: Code restructure failed: missing block: B:24:?, code lost:
    
        return;
     */
    /* JADX WARN: Code restructure failed: missing block: B:27:0x007e, code lost:
    
        if (r9 == null) goto L17;
     */
    /* JADX WARN: Code restructure failed: missing block: B:28:0x0081, code lost:
    
        r9.close();
     */
    /* JADX WARN: Code restructure failed: missing block: B:30:0x0089, code lost:
    
        if (r8 == null) goto L20;
     */
    /* JADX WARN: Code restructure failed: missing block: B:31:0x008c, code lost:
    
        r8.close();
     */
    /* JADX WARN: Code restructure failed: missing block: B:33:0x0079, code lost:
    
        throw r11;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    public void explainSlowQuery(byte[] r6, java.lang.String r7) throws java.sql.SQLException {
        /*
            r5 = this;
            r0 = r7
            java.lang.String r1 = "SELECT"
            boolean r0 = com.mysql.jdbc.StringUtils.startsWithIgnoreCaseAndWs(r0, r1)
            if (r0 == 0) goto L92
            r0 = 0
            r8 = r0
            r0 = 0
            r9 = r0
            r0 = r5
            com.mysql.jdbc.ConnectionImpl r0 = r0.connection     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            java.lang.String r1 = "EXPLAIN ?"
            com.mysql.jdbc.PreparedStatement r0 = r0.clientPrepareStatement(r1)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r8 = r0
            r0 = r8
            r1 = 1
            r2 = r6
            r0.setBytesNoEscapeNoQuotes(r1, r2)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r0 = r8
            java.sql.ResultSet r0 = r0.executeQuery()     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r9 = r0
            java.lang.StringBuffer r0 = new java.lang.StringBuffer     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r1 = r0
            java.lang.StringBuffer r2 = new java.lang.StringBuffer     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r3 = r2
            r3.<init>()     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            java.lang.String r3 = "MysqlIO.8"
            java.lang.String r3 = com.mysql.jdbc.Messages.getString(r3)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            java.lang.StringBuffer r2 = r2.append(r3)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r3 = r7
            java.lang.StringBuffer r2 = r2.append(r3)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            java.lang.String r3 = "MysqlIO.9"
            java.lang.String r3 = com.mysql.jdbc.Messages.getString(r3)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            java.lang.StringBuffer r2 = r2.append(r3)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            java.lang.String r2 = r2.toString()     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r1.<init>(r2)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r10 = r0
            r0 = r10
            r1 = r9
            java.lang.StringBuffer r0 = com.mysql.jdbc.util.ResultSetUtil.appendResultSetSlashGStyle(r0, r1)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r0 = r5
            com.mysql.jdbc.ConnectionImpl r0 = r0.connection     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            com.mysql.jdbc.log.Log r0 = r0.getLog()     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r1 = r10
            java.lang.String r1 = r1.toString()     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r0.logWarn(r1)     // Catch: java.sql.SQLException -> L6a java.lang.Throwable -> L72
            r0 = jsr -> L7a
        L67:
            goto L92
        L6a:
            r10 = move-exception
            r0 = jsr -> L7a
        L6f:
            goto L92
        L72:
            r11 = move-exception
            r0 = jsr -> L7a
        L77:
            r1 = r11
            throw r1
        L7a:
            r12 = r0
            r0 = r9
            if (r0 == 0) goto L88
            r0 = r9
            r0.close()
        L88:
            r0 = r8
            if (r0 == 0) goto L90
            r0 = r8
            r0.close()
        L90:
            ret r12
        L92:
            return
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.MysqlIO.explainSlowQuery(byte[], java.lang.String):void");
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public static int getMaxBuf() {
        return maxBufferSize;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public final int getServerMajorVersion() {
        return this.serverMajorVersion;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public final int getServerMinorVersion() {
        return this.serverMinorVersion;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public final int getServerSubMinorVersion() {
        return this.serverSubMinorVersion;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String getServerVersion() {
        return this.serverVersion;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void doHandshake(String user, String password, String database) throws SQLException {
        this.checkPacketSequence = false;
        this.readPacketSequence = (byte) 0;
        Buffer buf = readPacket();
        this.protocolVersion = buf.readByte();
        if (this.protocolVersion == -1) {
            try {
                this.mysqlConnection.close();
            } catch (Exception e) {
            }
            int errno = buf.readInt();
            String serverErrorMessage = buf.readString();
            StringBuffer errorBuf = new StringBuffer(Messages.getString("MysqlIO.10"));
            errorBuf.append(serverErrorMessage);
            errorBuf.append("\"");
            String xOpen = SQLError.mysqlToSqlState(errno, this.connection.getUseSqlStateCodes());
            throw SQLError.createSQLException(new StringBuffer().append(SQLError.get(xOpen)).append(", ").append(errorBuf.toString()).toString(), xOpen, errno);
        }
        this.serverVersion = buf.readString();
        int point = this.serverVersion.indexOf(46);
        if (point != -1) {
            try {
                int n = Integer.parseInt(this.serverVersion.substring(0, point));
                this.serverMajorVersion = n;
            } catch (NumberFormatException e2) {
            }
            String remaining = this.serverVersion.substring(point + 1, this.serverVersion.length());
            int point2 = remaining.indexOf(46);
            if (point2 != -1) {
                try {
                    int n2 = Integer.parseInt(remaining.substring(0, point2));
                    this.serverMinorVersion = n2;
                } catch (NumberFormatException e3) {
                }
                String remaining2 = remaining.substring(point2 + 1, remaining.length());
                int pos = 0;
                while (pos < remaining2.length() && remaining2.charAt(pos) >= '0' && remaining2.charAt(pos) <= '9') {
                    pos++;
                }
                try {
                    int n3 = Integer.parseInt(remaining2.substring(0, pos));
                    this.serverSubMinorVersion = n3;
                } catch (NumberFormatException e4) {
                }
            }
        }
        if (versionMeetsMinimum(4, 0, 8)) {
            this.maxThreeBytes = 16777215;
            this.useNewLargePackets = true;
        } else {
            this.maxThreeBytes = 16581375;
            this.useNewLargePackets = false;
        }
        this.colDecimalNeedsBump = versionMeetsMinimum(COMP_HEADER_LENGTH, 23, 0);
        this.colDecimalNeedsBump = !versionMeetsMinimum(COMP_HEADER_LENGTH, 23, 15);
        this.useNewUpdateCounts = versionMeetsMinimum(COMP_HEADER_LENGTH, 22, 5);
        this.threadId = buf.readLong();
        this.seed = buf.readString();
        this.serverCapabilities = 0;
        if (buf.getPosition() < buf.getBufLength()) {
            this.serverCapabilities = buf.readInt();
        }
        if (versionMeetsMinimum(4, 1, 1)) {
            int position = buf.getPosition();
            this.serverCharsetIndex = buf.readByte() & 255;
            this.serverStatus = buf.readInt();
            buf.setPosition(position + SERVER_QUERY_NO_GOOD_INDEX_USED);
            String seedPart2 = buf.readString();
            StringBuffer newSeed = new StringBuffer(20);
            newSeed.append(this.seed);
            newSeed.append(seedPart2);
            this.seed = newSeed.toString();
        }
        if ((this.serverCapabilities & 32) != 0 && this.connection.getUseCompression()) {
            this.clientParam |= 32;
        }
        this.useConnectWithDb = (database == null || database.length() <= 0 || this.connection.getCreateDatabaseIfNotExist()) ? false : true;
        if (this.useConnectWithDb) {
            this.clientParam |= 8;
        }
        if ((this.serverCapabilities & CLIENT_SSL) == 0 && this.connection.getUseSSL()) {
            if (this.connection.getRequireSSL()) {
                this.connection.close();
                forceClose();
                throw SQLError.createSQLException(Messages.getString("MysqlIO.15"), SQLError.SQL_STATE_UNABLE_TO_CONNECT_TO_DATASOURCE);
            }
            this.connection.setUseSSL(false);
        }
        if ((this.serverCapabilities & 4) != 0) {
            this.clientParam |= 4;
            this.hasLongColumnInfo = true;
        }
        this.clientParam |= 2;
        if (this.connection.getAllowLoadLocalInfile()) {
            this.clientParam |= 128;
        }
        if (this.isInteractiveClient) {
            this.clientParam |= 1024;
        }
        if (this.protocolVersion > 9) {
            this.clientParam |= 1;
        } else {
            this.clientParam &= -2;
        }
        if (versionMeetsMinimum(4, 1, 0)) {
            if (versionMeetsMinimum(4, 1, 1)) {
                this.clientParam |= 512;
                this.has41NewNewProt = true;
                this.clientParam |= 8192;
                this.clientParam |= 131072;
                if (this.connection.getAllowMultiQueries()) {
                    this.clientParam |= 65536;
                }
            } else {
                this.clientParam |= 16384;
                this.has41NewNewProt = false;
            }
            this.use41Extensions = true;
        }
        int userLength = user != null ? user.length() : 0;
        int databaseLength = database != null ? database.length() : 0;
        int packLength = ((userLength + SERVER_QUERY_NO_GOOD_INDEX_USED + databaseLength) * 2) + 7 + 4 + AUTH_411_OVERHEAD;
        if (!this.connection.getUseSSL()) {
            if ((this.serverCapabilities & CLIENT_SECURE_CONNECTION) != 0) {
                this.clientParam |= 32768;
                if (versionMeetsMinimum(4, 1, 1)) {
                    secureAuth411(null, packLength, user, password, database, true);
                } else {
                    secureAuth(null, packLength, user, password, database, true);
                }
            } else {
                Buffer packet = new Buffer(packLength);
                if ((this.clientParam & 16384) != 0) {
                    if (versionMeetsMinimum(4, 1, 1)) {
                        packet.writeLong(this.clientParam);
                        packet.writeLong(this.maxThreeBytes);
                        packet.writeByte((byte) 8);
                        packet.writeBytesNoNull(new byte[23]);
                    } else {
                        packet.writeLong(this.clientParam);
                        packet.writeLong(this.maxThreeBytes);
                    }
                } else {
                    packet.writeInt((int) this.clientParam);
                    packet.writeLongInt(this.maxThreeBytes);
                }
                packet.writeString(user, CODE_PAGE_1252, this.connection);
                if (this.protocolVersion > 9) {
                    packet.writeString(Util.newCrypt(password, this.seed), CODE_PAGE_1252, this.connection);
                } else {
                    packet.writeString(Util.oldCrypt(password, this.seed), CODE_PAGE_1252, this.connection);
                }
                if (this.useConnectWithDb) {
                    packet.writeString(database, CODE_PAGE_1252, this.connection);
                }
                send(packet, packet.getPosition());
            }
        } else {
            negotiateSSLConnection(user, password, database, packLength);
        }
        if (!versionMeetsMinimum(4, 1, 1)) {
            checkErrorPacket();
        }
        if ((this.serverCapabilities & 32) != 0 && this.connection.getUseCompression()) {
            this.deflater = new Deflater();
            this.useCompression = true;
            this.mysqlInput = new CompressedInputStream(this.connection, this.mysqlInput);
        }
        if (!this.useConnectWithDb) {
            changeDatabaseTo(database);
        }
    }

    private void changeDatabaseTo(String database) throws SQLException {
        if (database == null || database.length() == 0) {
            return;
        }
        try {
            sendCommand(2, database, null, false, null);
        } catch (Exception ex) {
            if (this.connection.getCreateDatabaseIfNotExist()) {
                sendCommand(COMP_HEADER_LENGTH, new StringBuffer().append("CREATE DATABASE IF NOT EXISTS ").append(database).toString(), null, false, null);
                sendCommand(2, database, null, false, null);
                return;
            }
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ex);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Type inference failed for: r0v27, types: [byte[], byte[][]] */
    public final ResultSetRow nextRow(Field[] fields, int columnCount, boolean isBinaryEncoded, int resultSetConcurrency, boolean useBufferRowIfPossible, boolean useBufferRowExplicit, boolean canReuseRowPacketForBufferRow, Buffer existingRowPacket) throws SQLException {
        Buffer rowPacket;
        if (this.useDirectRowUnpack && existingRowPacket == null && !isBinaryEncoded && !useBufferRowIfPossible && !useBufferRowExplicit) {
            return nextRowFast(fields, columnCount, isBinaryEncoded, resultSetConcurrency, useBufferRowIfPossible, useBufferRowExplicit, canReuseRowPacketForBufferRow);
        }
        if (existingRowPacket == null) {
            rowPacket = checkErrorPacket();
            if (!useBufferRowExplicit && useBufferRowIfPossible && rowPacket.getBufLength() > this.useBufferRowSizeThreshold) {
                useBufferRowExplicit = true;
            }
        } else {
            rowPacket = existingRowPacket;
            checkErrorPacket(existingRowPacket);
        }
        if (!isBinaryEncoded) {
            rowPacket.setPosition(rowPacket.getPosition() - 1);
            if (!rowPacket.isLastDataPacket()) {
                if (resultSetConcurrency == 1008 || (!useBufferRowIfPossible && !useBufferRowExplicit)) {
                    ?? r0 = new byte[columnCount];
                    for (int i = 0; i < columnCount; i++) {
                        r0[i] = rowPacket.readLenByteArray(0);
                    }
                    return new ByteArrayRow(r0);
                }
                if (!canReuseRowPacketForBufferRow) {
                    this.reusablePacket = new Buffer(rowPacket.getBufLength());
                }
                return new BufferRow(rowPacket, fields, false);
            }
            readServerStatusForResultSets(rowPacket);
            return null;
        }
        if (!rowPacket.isLastDataPacket()) {
            if (resultSetConcurrency == 1008 || (!useBufferRowIfPossible && !useBufferRowExplicit)) {
                return unpackBinaryResultSetRow(fields, rowPacket, resultSetConcurrency);
            }
            if (!canReuseRowPacketForBufferRow) {
                this.reusablePacket = new Buffer(rowPacket.getBufLength());
            }
            return new BufferRow(rowPacket, fields, true);
        }
        rowPacket.setPosition(rowPacket.getPosition() - 1);
        readServerStatusForResultSets(rowPacket);
        return null;
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r0v73, types: [byte[]] */
    final ResultSetRow nextRowFast(Field[] fields, int columnCount, boolean isBinaryEncoded, int resultSetConcurrency, boolean useBufferRowIfPossible, boolean useBufferRowExplicit, boolean canReuseRowPacket) throws SQLException {
        int len;
        try {
            int lengthRead = readFully(this.mysqlInput, this.packetHeaderBuf, 0, 4);
            if (lengthRead < 4) {
                forceClose();
                throw new RuntimeException(Messages.getString("MysqlIO.43"));
            }
            int packetLength = (this.packetHeaderBuf[0] & 255) + ((this.packetHeaderBuf[1] & 255) << 8) + ((this.packetHeaderBuf[2] & 255) << SERVER_QUERY_NO_GOOD_INDEX_USED);
            if (packetLength == this.maxThreeBytes) {
                reuseAndReadPacket(this.reusablePacket, packetLength);
                return nextRow(fields, columnCount, isBinaryEncoded, resultSetConcurrency, useBufferRowIfPossible, useBufferRowExplicit, canReuseRowPacket, this.reusablePacket);
            }
            if (packetLength > this.useBufferRowSizeThreshold) {
                reuseAndReadPacket(this.reusablePacket, packetLength);
                return nextRow(fields, columnCount, isBinaryEncoded, resultSetConcurrency, true, true, false, this.reusablePacket);
            }
            int remaining = packetLength;
            boolean firstTime = true;
            byte[][] rowData = (byte[][]) null;
            for (int i = 0; i < columnCount; i++) {
                int sw = this.mysqlInput.read() & 255;
                remaining--;
                if (firstTime) {
                    if (sw == 255) {
                        Buffer errorPacket = new Buffer(packetLength);
                        errorPacket.writeByte((byte) sw);
                        readFully(this.mysqlInput, errorPacket.getByteBuffer(), 1, packetLength - 1);
                        checkErrorPacket(errorPacket);
                    }
                    if (sw == 254 && packetLength < 9) {
                        this.warningCount = (this.mysqlInput.read() & 255) | ((this.mysqlInput.read() & 255) << 8);
                        int remaining2 = remaining - 2;
                        if (this.warningCount > 0) {
                            this.hadWarnings = true;
                        }
                        this.serverStatus = (this.mysqlInput.read() & 255) | ((this.mysqlInput.read() & 255) << 8);
                        int remaining3 = remaining2 - 2;
                        if (remaining3 > 0) {
                            skipFully(this.mysqlInput, remaining3);
                            return null;
                        }
                        return null;
                    }
                    rowData = new byte[columnCount];
                    firstTime = false;
                }
                switch (sw) {
                    case 251:
                        len = -1;
                        break;
                    case MysqlDefs.FIELD_TYPE_BLOB /* 252 */:
                        len = (this.mysqlInput.read() & 255) | ((this.mysqlInput.read() & 255) << 8);
                        remaining -= 2;
                        break;
                    case 253:
                        len = (this.mysqlInput.read() & 255) | ((this.mysqlInput.read() & 255) << 8) | ((this.mysqlInput.read() & 255) << SERVER_QUERY_NO_GOOD_INDEX_USED);
                        remaining -= 3;
                        break;
                    case 254:
                        len = (int) ((this.mysqlInput.read() & 255) | ((this.mysqlInput.read() & 255) << 8) | ((this.mysqlInput.read() & 255) << 16) | ((this.mysqlInput.read() & 255) << 24) | ((this.mysqlInput.read() & 255) << 32) | ((this.mysqlInput.read() & 255) << 40) | ((this.mysqlInput.read() & 255) << 48) | ((this.mysqlInput.read() & 255) << 56));
                        remaining -= 8;
                        break;
                    default:
                        len = sw;
                        break;
                }
                if (len == -1) {
                    rowData[i] = null;
                } else if (len == 0) {
                    rowData[i] = Constants.EMPTY_BYTE_ARRAY;
                } else {
                    rowData[i] = new byte[len];
                    int bytesRead = readFully(this.mysqlInput, rowData[i], 0, len);
                    if (bytesRead != len) {
                        throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, new IOException(Messages.getString("MysqlIO.43")));
                    }
                    remaining -= bytesRead;
                }
            }
            if (remaining > 0) {
                skipFully(this.mysqlInput, remaining);
            }
            return new ByteArrayRow(rowData);
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public final void quit() throws SQLException {
        Buffer packet = new Buffer(6);
        this.packetSequence = (byte) -1;
        packet.writeByte((byte) 1);
        send(packet, packet.getPosition());
        forceClose();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public Buffer getSharedSendPacket() {
        if (this.sharedSendPacket == null) {
            this.sharedSendPacket = new Buffer(MysqlErrorNumbers.ER_ERROR_ON_READ);
        }
        return this.sharedSendPacket;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void closeStreamer(RowData streamer) throws SQLException {
        if (this.streamingData == null) {
            throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.17")).append(streamer).append(Messages.getString("MysqlIO.18")).toString());
        }
        if (streamer != this.streamingData) {
            throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.19")).append(streamer).append(Messages.getString("MysqlIO.20")).append(Messages.getString("MysqlIO.21")).append(Messages.getString("MysqlIO.22")).toString());
        }
        this.streamingData = null;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Incorrect condition in loop: B:16:0x0070 */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    public com.mysql.jdbc.ResultSetImpl readAllResults(com.mysql.jdbc.StatementImpl r14, int r15, int r16, int r17, boolean r18, java.lang.String r19, com.mysql.jdbc.Buffer r20, boolean r21, long r22, com.mysql.jdbc.Field[] r24) throws java.sql.SQLException {
        /*
            r13 = this;
            r0 = r20
            r1 = r20
            int r1 = r1.getPosition()
            r2 = 1
            int r1 = r1 - r2
            r0.setPosition(r1)
            r0 = r13
            r1 = r14
            r2 = r15
            r3 = r16
            r4 = r17
            r5 = r18
            r6 = r19
            r7 = r20
            r8 = r21
            r9 = r22
            r10 = r24
            com.mysql.jdbc.ResultSetImpl r0 = r0.readResultsForQueryOrUpdate(r1, r2, r3, r4, r5, r6, r7, r8, r9, r10)
            r25 = r0
            r0 = r25
            r26 = r0
            r0 = r13
            long r0 = r0.clientParam
            r1 = 131072(0x20000, double:6.4758E-319)
            long r0 = r0 & r1
            r1 = 0
            int r0 = (r0 > r1 ? 1 : (r0 == r1 ? 0 : -1))
            if (r0 == 0) goto L38
            r0 = 1
            goto L39
        L38:
            r0 = 0
        L39:
            r27 = r0
            r0 = r13
            int r0 = r0.serverStatus
            r1 = 8
            r0 = r0 & r1
            if (r0 == 0) goto L49
            r0 = 1
            goto L4a
        L49:
            r0 = 0
        L4a:
            r28 = r0
            r0 = r28
            if (r0 == 0) goto L67
            r0 = r18
            if (r0 == 0) goto L67
            r0 = r13
            r0.clearInputStream()
            java.lang.String r0 = "MysqlIO.23"
            java.lang.String r0 = com.mysql.jdbc.Messages.getString(r0)
            java.lang.String r1 = "S1C00"
            java.sql.SQLException r0 = com.mysql.jdbc.SQLError.createSQLException(r0, r1)
            throw r0
        L67:
            r0 = r27
            r1 = r28
            r0 = r0 & r1
            r29 = r0
        L6e:
            r0 = r29
            if (r0 == 0) goto Lb5
            r0 = r13
            com.mysql.jdbc.Buffer r0 = r0.checkErrorPacket()
            r30 = r0
            r0 = r30
            r1 = 0
            r0.setPosition(r1)
            r0 = r13
            r1 = r14
            r2 = r15
            r3 = r16
            r4 = r17
            r5 = r18
            r6 = r19
            r7 = r30
            r8 = r21
            r9 = r22
            r10 = r24
            com.mysql.jdbc.ResultSetImpl r0 = r0.readResultsForQueryOrUpdate(r1, r2, r3, r4, r5, r6, r7, r8, r9, r10)
            r31 = r0
            r0 = r26
            r1 = r31
            r0.setNextResultSet(r1)
            r0 = r31
            r26 = r0
            r0 = r13
            int r0 = r0.serverStatus
            r1 = 8
            r0 = r0 & r1
            if (r0 == 0) goto Laf
            r0 = 1
            goto Lb0
        Laf:
            r0 = 0
        Lb0:
            r29 = r0
            goto L6e
        Lb5:
            r0 = r18
            if (r0 != 0) goto Lbe
            r0 = r13
            r0.clearInputStream()
        Lbe:
            r0 = r13
            r0.reclaimLargeReusablePacket()
            r0 = r25
            return r0
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.MysqlIO.readAllResults(com.mysql.jdbc.StatementImpl, int, int, int, boolean, java.lang.String, com.mysql.jdbc.Buffer, boolean, long, com.mysql.jdbc.Field[]):com.mysql.jdbc.ResultSetImpl");
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void resetMaxBuf() {
        this.maxAllowedPacket = this.connection.getMaxAllowedPacket();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public final Buffer sendCommand(int command, String extraData, Buffer queryPacket, boolean skipCheck, String extraDataCharEncoding) throws SQLException {
        int bytesLeft;
        this.enablePacketDebug = this.connection.getEnablePacketDebug();
        this.traceProtocol = this.connection.getTraceProtocol();
        this.readPacketSequence = (byte) 0;
        try {
            checkForOutstandingStreamingData();
            this.serverStatus = 0;
            this.hadWarnings = false;
            this.warningCount = 0;
            this.queryNoIndexUsed = false;
            this.queryBadIndexUsed = false;
            if (this.useCompression && (bytesLeft = this.mysqlInput.available()) > 0) {
                this.mysqlInput.skip(bytesLeft);
            }
            try {
                clearInputStream();
                if (queryPacket == null) {
                    int packLength = 8 + (extraData != null ? extraData.length() : 0) + 2;
                    if (this.sendPacket == null) {
                        this.sendPacket = new Buffer(packLength);
                    }
                    this.packetSequence = (byte) -1;
                    this.readPacketSequence = (byte) 0;
                    this.checkPacketSequence = true;
                    this.sendPacket.clear();
                    this.sendPacket.writeByte((byte) command);
                    if (command == 2 || command == 5 || command == 6 || command == COMP_HEADER_LENGTH || command == 22) {
                        if (extraDataCharEncoding == null) {
                            this.sendPacket.writeStringNoNull(extraData);
                        } else {
                            this.sendPacket.writeStringNoNull(extraData, extraDataCharEncoding, this.connection.getServerCharacterEncoding(), this.connection.parserKnowsUnicode(), this.connection);
                        }
                    } else if (command == 12) {
                        long id = Long.parseLong(extraData);
                        this.sendPacket.writeLong(id);
                    }
                    send(this.sendPacket, this.sendPacket.getPosition());
                } else {
                    this.packetSequence = (byte) -1;
                    send(queryPacket, queryPacket.getPosition());
                }
                Buffer returnPacket = null;
                if (!skipCheck) {
                    if (command == 23 || command == 26) {
                        this.readPacketSequence = (byte) 0;
                        this.packetSequenceReset = true;
                    }
                    returnPacket = checkErrorPacket(command);
                }
                return returnPacket;
            } catch (SQLException sqlEx) {
                throw sqlEx;
            } catch (Exception ex) {
                throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ex);
            }
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public final ResultSetInternalMethods sqlQueryDirect(StatementImpl callingStatement, String query, String characterEncoding, Buffer queryPacket, int maxRows, int resultSetType, int resultSetConcurrency, boolean streamResults, String catalog, Field[] cachedMetadata) throws Exception {
        String testcaseQuery;
        ResultSetInternalMethods interceptedResults;
        this.statementExecutionDepth++;
        try {
            if (this.statementInterceptors != null && (interceptedResults = invokeStatementInterceptorsPre(query, callingStatement)) != null) {
                return interceptedResults;
            }
            long queryStartTime = 0;
            long queryEndTime = 0;
            if (query != null) {
                int packLength = 5 + (query.length() * 2) + 2;
                String statementComment = this.connection.getStatementComment();
                byte[] commentAsBytes = null;
                if (statementComment != null) {
                    commentAsBytes = StringUtils.getBytes(statementComment, (SingleByteCharsetConverter) null, characterEncoding, this.connection.getServerCharacterEncoding(), this.connection.parserKnowsUnicode());
                    packLength = packLength + commentAsBytes.length + 6;
                }
                if (this.sendPacket == null) {
                    this.sendPacket = new Buffer(packLength);
                } else {
                    this.sendPacket.clear();
                }
                this.sendPacket.writeByte((byte) 3);
                if (commentAsBytes != null) {
                    this.sendPacket.writeBytesNoNull(Constants.SLASH_STAR_SPACE_AS_BYTES);
                    this.sendPacket.writeBytesNoNull(commentAsBytes);
                    this.sendPacket.writeBytesNoNull(Constants.SPACE_STAR_SLASH_SPACE_AS_BYTES);
                }
                if (characterEncoding != null) {
                    if (this.platformDbCharsetMatches) {
                        this.sendPacket.writeStringNoNull(query, characterEncoding, this.connection.getServerCharacterEncoding(), this.connection.parserKnowsUnicode(), this.connection);
                    } else if (StringUtils.startsWithIgnoreCaseAndWs(query, "LOAD DATA")) {
                        this.sendPacket.writeBytesNoNull(query.getBytes());
                    } else {
                        this.sendPacket.writeStringNoNull(query, characterEncoding, this.connection.getServerCharacterEncoding(), this.connection.parserKnowsUnicode(), this.connection);
                    }
                } else {
                    this.sendPacket.writeStringNoNull(query);
                }
                queryPacket = this.sendPacket;
            }
            byte[] queryBuf = null;
            int oldPacketPosition = 0;
            if (this.needToGrabQueryFromPacket) {
                queryBuf = queryPacket.getByteBuffer();
                oldPacketPosition = queryPacket.getPosition();
                queryStartTime = getCurrentTimeNanosOrMillis();
            }
            Buffer resultPacket = sendCommand(COMP_HEADER_LENGTH, null, queryPacket, false, null);
            long fetchBeginTime = 0;
            String profileQueryToLog = null;
            boolean queryWasSlow = false;
            if (this.profileSql || this.logSlowQueries) {
                queryEndTime = System.currentTimeMillis();
                boolean shouldExtractQuery = false;
                if (this.profileSql) {
                    shouldExtractQuery = true;
                } else if (this.logSlowQueries && queryEndTime - queryStartTime > this.connection.getSlowQueryThresholdMillis()) {
                    shouldExtractQuery = true;
                    queryWasSlow = true;
                }
                if (shouldExtractQuery) {
                    boolean truncated = false;
                    int extractPosition = oldPacketPosition;
                    if (oldPacketPosition > this.connection.getMaxQuerySizeToLog()) {
                        extractPosition = this.connection.getMaxQuerySizeToLog() + 5;
                        truncated = true;
                    }
                    profileQueryToLog = new String(queryBuf, 5, extractPosition - 5);
                    if (truncated) {
                        profileQueryToLog = new StringBuffer().append(profileQueryToLog).append(Messages.getString("MysqlIO.25")).toString();
                    }
                }
                fetchBeginTime = queryEndTime;
            }
            if (this.autoGenerateTestcaseScript) {
                if (query != null) {
                    testcaseQuery = query;
                } else {
                    testcaseQuery = new String(queryBuf, 5, oldPacketPosition - 5);
                }
                StringBuffer debugBuf = new StringBuffer(testcaseQuery.length() + 32);
                this.connection.generateConnectionCommentBlock(debugBuf);
                debugBuf.append(testcaseQuery);
                debugBuf.append(';');
                this.connection.dumpTestcaseQuery(debugBuf.toString());
            }
            ResultSetInternalMethods rs = readAllResults(callingStatement, maxRows, resultSetType, resultSetConcurrency, streamResults, catalog, resultPacket, false, -1L, cachedMetadata);
            if (queryWasSlow) {
                StringBuffer mesgBuf = new StringBuffer(48 + profileQueryToLog.length());
                mesgBuf.append(Messages.getString("MysqlIO.SlowQuery", new Object[]{new Long(this.slowQueryThreshold), this.queryTimingUnits, new Long(queryEndTime - queryStartTime)}));
                mesgBuf.append(profileQueryToLog);
                ProfileEventSink.getInstance(this.connection).consumeEvent(new ProfilerEvent((byte) 6, "", catalog, this.connection.getId(), callingStatement != null ? callingStatement.getId() : 999, ((ResultSetImpl) rs).resultId, System.currentTimeMillis(), (int) (queryEndTime - queryStartTime), this.queryTimingUnits, (String) null, new Throwable(), mesgBuf.toString()));
                if (this.connection.getExplainSlowQueries()) {
                    if (oldPacketPosition < MAX_QUERY_SIZE_TO_EXPLAIN) {
                        explainSlowQuery(queryPacket.getBytes(5, oldPacketPosition - 5), profileQueryToLog);
                    } else {
                        this.connection.getLog().logWarn(new StringBuffer().append(Messages.getString("MysqlIO.28")).append(MAX_QUERY_SIZE_TO_EXPLAIN).append(Messages.getString("MysqlIO.29")).toString());
                    }
                }
            }
            if (this.logSlowQueries) {
                ProfileEventSink eventSink = ProfileEventSink.getInstance(this.connection);
                if (this.queryBadIndexUsed) {
                    eventSink.consumeEvent(new ProfilerEvent((byte) 6, "", catalog, this.connection.getId(), callingStatement != null ? callingStatement.getId() : 999, ((ResultSetImpl) rs).resultId, System.currentTimeMillis(), queryEndTime - queryStartTime, this.queryTimingUnits, (String) null, new Throwable(), new StringBuffer().append(Messages.getString("MysqlIO.33")).append(profileQueryToLog).toString()));
                }
                if (this.queryNoIndexUsed) {
                    eventSink.consumeEvent(new ProfilerEvent((byte) 6, "", catalog, this.connection.getId(), callingStatement != null ? callingStatement.getId() : 999, ((ResultSetImpl) rs).resultId, System.currentTimeMillis(), queryEndTime - queryStartTime, this.queryTimingUnits, (String) null, new Throwable(), new StringBuffer().append(Messages.getString("MysqlIO.35")).append(profileQueryToLog).toString()));
                }
            }
            if (this.profileSql) {
                long fetchEndTime = getCurrentTimeNanosOrMillis();
                ProfileEventSink eventSink2 = ProfileEventSink.getInstance(this.connection);
                eventSink2.consumeEvent(new ProfilerEvent((byte) 3, "", catalog, this.connection.getId(), callingStatement != null ? callingStatement.getId() : 999, ((ResultSetImpl) rs).resultId, System.currentTimeMillis(), queryEndTime - queryStartTime, this.queryTimingUnits, (String) null, new Throwable(), profileQueryToLog));
                eventSink2.consumeEvent(new ProfilerEvent((byte) 5, "", catalog, this.connection.getId(), callingStatement != null ? callingStatement.getId() : 999, ((ResultSetImpl) rs).resultId, System.currentTimeMillis(), fetchEndTime - fetchBeginTime, this.queryTimingUnits, (String) null, new Throwable(), (String) null));
            }
            if (this.hadWarnings) {
                scanForAndThrowDataTruncation();
            }
            if (this.statementInterceptors != null) {
                ResultSetInternalMethods interceptedResults2 = invokeStatementInterceptorsPost(query, callingStatement, rs);
                if (interceptedResults2 != null) {
                    rs = interceptedResults2;
                }
            }
            ResultSetInternalMethods resultSetInternalMethods = rs;
            this.statementExecutionDepth--;
            return resultSetInternalMethods;
        } finally {
            this.statementExecutionDepth--;
        }
    }

    private ResultSetInternalMethods invokeStatementInterceptorsPre(String sql, Statement interceptedStatement) throws SQLException {
        ResultSetInternalMethods previousResultSet = null;
        for (StatementInterceptor interceptor : this.statementInterceptors) {
            boolean executeTopLevelOnly = interceptor.executeTopLevelOnly();
            boolean shouldExecute = (executeTopLevelOnly && this.statementExecutionDepth == 1) || !executeTopLevelOnly;
            if (shouldExecute) {
                String sqlToInterceptor = sql;
                if (interceptedStatement instanceof PreparedStatement) {
                    sqlToInterceptor = ((PreparedStatement) interceptedStatement).asSql();
                }
                ResultSetInternalMethods interceptedResultSet = interceptor.preProcess(sqlToInterceptor, interceptedStatement, this.connection);
                if (interceptedResultSet != null) {
                    previousResultSet = interceptedResultSet;
                }
            }
        }
        return previousResultSet;
    }

    private ResultSetInternalMethods invokeStatementInterceptorsPost(String sql, Statement interceptedStatement, ResultSetInternalMethods originalResultSet) throws SQLException {
        for (StatementInterceptor interceptor : this.statementInterceptors) {
            boolean executeTopLevelOnly = interceptor.executeTopLevelOnly();
            boolean shouldExecute = (executeTopLevelOnly && this.statementExecutionDepth == 1) || !executeTopLevelOnly;
            if (shouldExecute) {
                String sqlToInterceptor = sql;
                if (interceptedStatement instanceof PreparedStatement) {
                    sqlToInterceptor = ((PreparedStatement) interceptedStatement).asSql();
                }
                ResultSetInternalMethods interceptedResultSet = interceptor.postProcess(sqlToInterceptor, interceptedStatement, originalResultSet, this.connection);
                if (interceptedResultSet != null) {
                    originalResultSet = interceptedResultSet;
                }
            }
        }
        return originalResultSet;
    }

    private void calculateSlowQueryThreshold() {
        this.slowQueryThreshold = this.connection.getSlowQueryThresholdMillis();
        if (this.connection.getUseNanosForElapsedTime()) {
            long nanosThreshold = this.connection.getSlowQueryThresholdNanos();
            if (nanosThreshold != 0) {
                this.slowQueryThreshold = nanosThreshold;
            } else {
                this.slowQueryThreshold *= 1000000;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public long getCurrentTimeNanosOrMillis() {
        if (this.useNanosForElapsedTime) {
            return Util.getCurrentTimeNanosOrMillis();
        }
        return System.currentTimeMillis();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String getHost() {
        return this.host;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public boolean isVersion(int major, int minor, int subminor) {
        return major == getServerMajorVersion() && minor == getServerMinorVersion() && subminor == getServerSubMinorVersion();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public boolean versionMeetsMinimum(int major, int minor, int subminor) {
        if (getServerMajorVersion() >= major) {
            if (getServerMajorVersion() == major) {
                if (getServerMinorVersion() >= minor) {
                    return getServerMinorVersion() != minor || getServerSubMinorVersion() >= subminor;
                }
                return false;
            }
            return true;
        }
        return false;
    }

    private static final String getPacketDumpToLog(Buffer packetToDump, int packetLength) {
        if (packetLength < 1024) {
            return packetToDump.dump(packetLength);
        }
        StringBuffer packetDumpBuf = new StringBuffer(4096);
        packetDumpBuf.append(packetToDump.dump(MysqlErrorNumbers.ER_ERROR_ON_READ));
        packetDumpBuf.append(Messages.getString("MysqlIO.36"));
        packetDumpBuf.append(MysqlErrorNumbers.ER_ERROR_ON_READ);
        packetDumpBuf.append(Messages.getString("MysqlIO.37"));
        return packetDumpBuf.toString();
    }

    private final int readFully(InputStream in, byte[] b, int off, int len) throws IOException {
        if (len < 0) {
            throw new IndexOutOfBoundsException();
        }
        int i = 0;
        while (true) {
            int n = i;
            if (n < len) {
                int count = in.read(b, off + n, len - n);
                if (count < 0) {
                    throw new EOFException(Messages.getString("MysqlIO.EOF", new Object[]{new Integer(len), new Integer(n)}));
                }
                i = n + count;
            } else {
                return n;
            }
        }
    }

    private final long skipFully(InputStream in, long len) throws IOException {
        if (len < 0) {
            throw new IOException("Negative skip length not allowed");
        }
        long j = 0;
        while (true) {
            long n = j;
            if (n < len) {
                long count = in.skip(len - n);
                if (count < 0) {
                    throw new EOFException(Messages.getString("MysqlIO.EOF", new Object[]{new Long(len), new Long(n)}));
                }
                j = n + count;
            } else {
                return n;
            }
        }
    }

    private final ResultSetImpl readResultsForQueryOrUpdate(StatementImpl callingStatement, int maxRows, int resultSetType, int resultSetConcurrency, boolean streamResults, String catalog, Buffer resultPacket, boolean isBinaryEncoded, long preSentColumnCount, Field[] metadataFromCache) throws SQLException {
        String fileName;
        long columnCount = resultPacket.readFieldLength();
        if (columnCount == 0) {
            return buildResultSetWithUpdates(callingStatement, resultPacket);
        }
        if (columnCount == -1) {
            String charEncoding = null;
            if (this.connection.getUseUnicode()) {
                charEncoding = this.connection.getEncoding();
            }
            if (this.platformDbCharsetMatches) {
                fileName = charEncoding != null ? resultPacket.readString(charEncoding) : resultPacket.readString();
            } else {
                fileName = resultPacket.readString();
            }
            return sendFileToServer(callingStatement, fileName);
        }
        ResultSetImpl results = getResultSet(callingStatement, columnCount, maxRows, resultSetType, resultSetConcurrency, streamResults, catalog, isBinaryEncoded, metadataFromCache);
        return results;
    }

    private int alignPacketSize(int a, int l) {
        return ((a + l) - 1) & ((l - 1) ^ (-1));
    }

    private ResultSetImpl buildResultSetWithRows(StatementImpl callingStatement, String catalog, Field[] fields, RowData rows, int resultSetType, int resultSetConcurrency, boolean isBinaryEncoded) throws SQLException {
        ResultSetImpl rs;
        switch (resultSetConcurrency) {
            case MysqlErrorNumbers.ER_DB_CREATE_EXISTS /* 1007 */:
                rs = ResultSetImpl.getInstance(catalog, fields, rows, this.connection, callingStatement, false);
                if (isBinaryEncoded) {
                    rs.setBinaryEncoded();
                    break;
                }
                break;
            case MysqlErrorNumbers.ER_DB_DROP_EXISTS /* 1008 */:
                rs = ResultSetImpl.getInstance(catalog, fields, rows, this.connection, callingStatement, true);
                break;
            default:
                return ResultSetImpl.getInstance(catalog, fields, rows, this.connection, callingStatement, false);
        }
        rs.setResultSetType(resultSetType);
        rs.setResultSetConcurrency(resultSetConcurrency);
        return rs;
    }

    private ResultSetImpl buildResultSetWithUpdates(StatementImpl callingStatement, Buffer resultPacket) throws SQLException {
        long updateCount;
        long updateID;
        String info = null;
        try {
            if (this.useNewUpdateCounts) {
                updateCount = resultPacket.newReadLength();
                updateID = resultPacket.newReadLength();
            } else {
                updateCount = resultPacket.readLength();
                updateID = resultPacket.readLength();
            }
            if (this.use41Extensions) {
                this.serverStatus = resultPacket.readInt();
                this.warningCount = resultPacket.readInt();
                if (this.warningCount > 0) {
                    this.hadWarnings = true;
                }
                resultPacket.readByte();
                if (this.profileSql) {
                    this.queryNoIndexUsed = (this.serverStatus & SERVER_QUERY_NO_GOOD_INDEX_USED) != 0;
                    this.queryBadIndexUsed = (this.serverStatus & 32) != 0;
                }
            }
            if (this.connection.isReadInfoMsgEnabled()) {
                info = resultPacket.readString();
            }
            ResultSetInternalMethods updateRs = ResultSetImpl.getInstance(updateCount, updateID, this.connection, callingStatement);
            if (info != null) {
                ((ResultSetImpl) updateRs).setServerInfo(info);
            }
            return (ResultSetImpl) updateRs;
        } catch (Exception ex) {
            throw SQLError.createSQLException(new StringBuffer().append(SQLError.get(SQLError.SQL_STATE_GENERAL_ERROR)).append(": ").append(ex.getClass().getName()).toString(), SQLError.SQL_STATE_GENERAL_ERROR, -1);
        }
    }

    private void checkForOutstandingStreamingData() throws SQLException {
        if (this.streamingData != null) {
            boolean shouldClobber = this.connection.getClobberStreamingResults();
            if (!shouldClobber) {
                throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.39")).append(this.streamingData).append(Messages.getString("MysqlIO.40")).append(Messages.getString("MysqlIO.41")).append(Messages.getString("MysqlIO.42")).toString());
            }
            this.streamingData.getOwner().realClose(false);
            clearInputStream();
        }
    }

    private Buffer compressPacket(Buffer packet, int offset, int packetLen, int headerLength) throws SQLException {
        byte[] compressedBytes;
        int lengthToWrite;
        int compressedLength;
        packet.writeLongInt(packetLen - headerLength);
        packet.writeByte((byte) 0);
        byte[] bytesToCompress = packet.getByteBuffer();
        int offsetWrite = 0;
        if (packetLen < MIN_COMPRESS_LEN) {
            lengthToWrite = packetLen;
            compressedBytes = packet.getByteBuffer();
            compressedLength = 0;
            offsetWrite = offset;
        } else {
            compressedBytes = new byte[bytesToCompress.length * 2];
            this.deflater.reset();
            this.deflater.setInput(bytesToCompress, offset, packetLen);
            this.deflater.finish();
            int compLen = this.deflater.deflate(compressedBytes);
            if (compLen > packetLen) {
                lengthToWrite = packetLen;
                compressedBytes = packet.getByteBuffer();
                compressedLength = 0;
                offsetWrite = offset;
            } else {
                lengthToWrite = compLen;
                headerLength += COMP_HEADER_LENGTH;
                compressedLength = packetLen;
            }
        }
        Buffer compressedPacket = new Buffer(packetLen + headerLength);
        compressedPacket.setPosition(0);
        compressedPacket.writeLongInt(lengthToWrite);
        compressedPacket.writeByte(this.packetSequence);
        compressedPacket.writeLongInt(compressedLength);
        compressedPacket.writeBytesNoNull(compressedBytes, offsetWrite, lengthToWrite);
        return compressedPacket;
    }

    private final void readServerStatusForResultSets(Buffer rowPacket) throws SQLException {
        if (this.use41Extensions) {
            rowPacket.readByte();
            this.warningCount = rowPacket.readInt();
            if (this.warningCount > 0) {
                this.hadWarnings = true;
            }
            this.serverStatus = rowPacket.readInt();
            if (this.profileSql) {
                this.queryNoIndexUsed = (this.serverStatus & SERVER_QUERY_NO_GOOD_INDEX_USED) != 0;
                this.queryBadIndexUsed = (this.serverStatus & 32) != 0;
            }
        }
    }

    private SocketFactory createSocketFactory() throws SQLException {
        try {
            if (this.socketFactoryClassName == null) {
                throw SQLError.createSQLException(Messages.getString("MysqlIO.75"), SQLError.SQL_STATE_UNABLE_TO_CONNECT_TO_DATASOURCE);
            }
            return (SocketFactory) Class.forName(this.socketFactoryClassName).newInstance();
        } catch (Exception ex) {
            throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.76")).append(this.socketFactoryClassName).append(Messages.getString("MysqlIO.77")).append(ex.toString()).append(this.connection.getParanoid() ? "" : Util.stackTraceToString(ex)).toString(), SQLError.SQL_STATE_UNABLE_TO_CONNECT_TO_DATASOURCE);
        }
    }

    private void enqueuePacketForDebugging(boolean isPacketBeingSent, boolean isPacketReused, int sendLength, byte[] header, Buffer packet) throws SQLException {
        StringBuffer packetDump;
        if (this.packetDebugRingBuffer.size() + 1 > this.connection.getPacketDebugBufferSize()) {
            this.packetDebugRingBuffer.removeFirst();
        }
        if (!isPacketBeingSent) {
            int bytesToDump = Math.min(MysqlErrorNumbers.ER_ERROR_ON_READ, packet.getBufLength());
            Buffer packetToDump = new Buffer(4 + bytesToDump);
            packetToDump.setPosition(0);
            packetToDump.writeBytesNoNull(header);
            packetToDump.writeBytesNoNull(packet.getBytes(0, bytesToDump));
            String packetPayload = packetToDump.dump(bytesToDump);
            packetDump = new StringBuffer(96 + packetPayload.length());
            packetDump.append("Server ");
            if (isPacketReused) {
                packetDump.append("(re-used)");
            } else {
                packetDump.append("(new)");
            }
            packetDump.append(" ");
            packetDump.append(packet.toSuperString());
            packetDump.append(" --------------------> Client\n");
            packetDump.append("\nPacket payload:\n\n");
            packetDump.append(packetPayload);
            if (bytesToDump == 1024) {
                packetDump.append(new StringBuffer().append("\nNote: Packet of ").append(packet.getBufLength()).append(" bytes truncated to ").append(MysqlErrorNumbers.ER_ERROR_ON_READ).append(" bytes.\n").toString());
            }
        } else {
            int bytesToDump2 = Math.min(MysqlErrorNumbers.ER_ERROR_ON_READ, sendLength);
            String packetPayload2 = packet.dump(bytesToDump2);
            packetDump = new StringBuffer(68 + packetPayload2.length());
            packetDump.append("Client ");
            packetDump.append(packet.toSuperString());
            packetDump.append("--------------------> Server\n");
            packetDump.append("\nPacket payload:\n\n");
            packetDump.append(packetPayload2);
            if (bytesToDump2 == 1024) {
                packetDump.append(new StringBuffer().append("\nNote: Packet of ").append(sendLength).append(" bytes truncated to ").append(MysqlErrorNumbers.ER_ERROR_ON_READ).append(" bytes.\n").toString());
            }
        }
        this.packetDebugRingBuffer.addLast(packetDump);
    }

    private RowData readSingleRowSet(long columnCount, int maxRows, int resultSetConcurrency, boolean isBinaryEncoded, Field[] fields) throws SQLException {
        ArrayList rows = new ArrayList();
        boolean useBufferRowExplicit = useBufferRowExplicit(fields);
        ResultSetRow row = nextRow(fields, (int) columnCount, isBinaryEncoded, resultSetConcurrency, false, useBufferRowExplicit, false, null);
        int rowCount = 0;
        if (row != null) {
            rows.add(row);
            rowCount = 1;
        }
        while (row != null) {
            row = nextRow(fields, (int) columnCount, isBinaryEncoded, resultSetConcurrency, false, useBufferRowExplicit, false, null);
            if (row != null && (maxRows == -1 || rowCount < maxRows)) {
                rows.add(row);
                rowCount++;
            }
        }
        RowData rowData = new RowDataStatic(rows);
        return rowData;
    }

    public static boolean useBufferRowExplicit(Field[] fields) {
        if (fields == null) {
            return false;
        }
        for (Field field : fields) {
            switch (field.getSQLType()) {
                case -4:
                case -1:
                case 2004:
                case 2005:
                    return true;
                default:
            }
        }
        return false;
    }

    private void reclaimLargeReusablePacket() {
        if (this.reusablePacket != null && this.reusablePacket.getCapacity() > MAX_QUERY_SIZE_TO_EXPLAIN) {
            this.reusablePacket = new Buffer(MysqlErrorNumbers.ER_ERROR_ON_READ);
        }
    }

    private final Buffer reuseAndReadPacket(Buffer reuse) throws SQLException {
        return reuseAndReadPacket(reuse, -1);
    }

    /* JADX WARN: Code restructure failed: missing block: B:54:0x01d5, code lost:
    
        r7.connection.realClose(false, false, true, r10);
     */
    /* JADX WARN: Code restructure failed: missing block: B:55:0x01e1, code lost:
    
        throw r10;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    private final com.mysql.jdbc.Buffer reuseAndReadPacket(com.mysql.jdbc.Buffer r8, int r9) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 489
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.MysqlIO.reuseAndReadPacket(com.mysql.jdbc.Buffer, int):com.mysql.jdbc.Buffer");
    }

    /* JADX WARN: Code restructure failed: missing block: B:18:0x020c, code lost:
    
        r7.setPosition(0);
        r7.setWasMultiPacket(true);
     */
    /* JADX WARN: Code restructure failed: missing block: B:19:0x0218, code lost:
    
        return r11;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    private int readRemainingMultiPackets(com.mysql.jdbc.Buffer r7, byte r8, int r9) throws java.io.IOException, java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 537
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.MysqlIO.readRemainingMultiPackets(com.mysql.jdbc.Buffer, byte, int):int");
    }

    private void checkPacketSequencing(byte multiPacketSeq) throws SQLException {
        if (multiPacketSeq == Byte.MIN_VALUE && this.readPacketSequence != Byte.MAX_VALUE) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, new IOException(new StringBuffer().append("Packets out of order, expected packet # -128, but received packet # ").append((int) multiPacketSeq).toString()));
        }
        if (this.readPacketSequence == -1 && multiPacketSeq != 0) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, new IOException(new StringBuffer().append("Packets out of order, expected packet # -1, but received packet # ").append((int) multiPacketSeq).toString()));
        }
        if (multiPacketSeq != Byte.MIN_VALUE && this.readPacketSequence != -1 && multiPacketSeq != this.readPacketSequence + 1) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, new IOException(new StringBuffer().append("Packets out of order, expected packet # ").append(this.readPacketSequence + 1).append(", but received packet # ").append((int) multiPacketSeq).toString()));
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void enableMultiQueries() throws SQLException {
        Buffer buf = getSharedSendPacket();
        buf.clear();
        buf.writeByte((byte) 27);
        buf.writeInt(0);
        sendCommand(27, null, buf, false, null);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void disableMultiQueries() throws SQLException {
        Buffer buf = getSharedSendPacket();
        buf.clear();
        buf.writeByte((byte) 27);
        buf.writeInt(1);
        sendCommand(27, null, buf, false, null);
    }

    private final void send(Buffer packet, int packetLen) throws SQLException {
        try {
            if (packetLen > this.maxAllowedPacket) {
                throw new PacketTooBigException(packetLen, this.maxAllowedPacket);
            }
            if (this.connection.getMaintainTimeStats()) {
                this.lastPacketSentTimeMs = System.currentTimeMillis();
            }
            if (this.serverMajorVersion >= 4 && packetLen >= this.maxThreeBytes) {
                sendSplitPackets(packet);
            } else {
                this.packetSequence = (byte) (this.packetSequence + 1);
                Buffer packetToSend = packet;
                packetToSend.setPosition(0);
                if (this.useCompression) {
                    packetToSend = compressPacket(packet, 0, packetLen, 4);
                    packetLen = packetToSend.getPosition();
                    if (this.traceProtocol) {
                        StringBuffer traceMessageBuf = new StringBuffer();
                        traceMessageBuf.append(Messages.getString("MysqlIO.57"));
                        traceMessageBuf.append(getPacketDumpToLog(packetToSend, packetLen));
                        traceMessageBuf.append(Messages.getString("MysqlIO.58"));
                        traceMessageBuf.append(getPacketDumpToLog(packet, packetLen));
                        this.connection.getLog().logTrace(traceMessageBuf.toString());
                    }
                } else {
                    packetToSend.writeLongInt(packetLen - 4);
                    packetToSend.writeByte(this.packetSequence);
                    if (this.traceProtocol) {
                        StringBuffer traceMessageBuf2 = new StringBuffer();
                        traceMessageBuf2.append(Messages.getString("MysqlIO.59"));
                        traceMessageBuf2.append(packetToSend.dump(packetLen));
                        this.connection.getLog().logTrace(traceMessageBuf2.toString());
                    }
                }
                this.mysqlOutput.write(packetToSend.getByteBuffer(), 0, packetLen);
                this.mysqlOutput.flush();
            }
            if (this.enablePacketDebug) {
                enqueuePacketForDebugging(true, false, packetLen + 5, this.packetHeaderBuf, packet);
            }
            if (packet == this.sharedSendPacket) {
                reclaimLargeSharedSendPacket();
            }
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        }
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:24:0x01e3
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    private final com.mysql.jdbc.ResultSetImpl sendFileToServer(com.mysql.jdbc.StatementImpl r7, java.lang.String r8) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 551
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.MysqlIO.sendFileToServer(com.mysql.jdbc.StatementImpl, java.lang.String):com.mysql.jdbc.ResultSetImpl");
    }

    private Buffer checkErrorPacket(int command) throws SQLException {
        this.serverStatus = 0;
        try {
            Buffer resultPacket = reuseAndReadPacket(this.reusablePacket);
            checkErrorPacket(resultPacket);
            return resultPacket;
        } catch (SQLException sqlEx) {
            throw sqlEx;
        } catch (Exception fallThru) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, fallThru);
        }
    }

    private void checkErrorPacket(Buffer resultPacket) throws SQLException {
        String xOpen;
        int statusCode = resultPacket.readByte();
        if (statusCode == -1) {
            if (this.protocolVersion > 9) {
                int errno = resultPacket.readInt();
                String serverErrorMessage = resultPacket.readString(this.connection.getErrorMessageEncoding());
                if (serverErrorMessage.charAt(0) == '#') {
                    if (serverErrorMessage.length() > 6) {
                        xOpen = serverErrorMessage.substring(1, 6);
                        serverErrorMessage = serverErrorMessage.substring(6);
                        if (xOpen.equals("HY000")) {
                            xOpen = SQLError.mysqlToSqlState(errno, this.connection.getUseSqlStateCodes());
                        }
                    } else {
                        xOpen = SQLError.mysqlToSqlState(errno, this.connection.getUseSqlStateCodes());
                    }
                } else {
                    xOpen = SQLError.mysqlToSqlState(errno, this.connection.getUseSqlStateCodes());
                }
                clearInputStream();
                StringBuffer errorBuf = new StringBuffer();
                String xOpenErrorMessage = SQLError.get(xOpen);
                if (!this.connection.getUseOnlyServerErrorMessages() && xOpenErrorMessage != null) {
                    errorBuf.append(xOpenErrorMessage);
                    errorBuf.append(Messages.getString("MysqlIO.68"));
                }
                errorBuf.append(serverErrorMessage);
                if (!this.connection.getUseOnlyServerErrorMessages() && xOpenErrorMessage != null) {
                    errorBuf.append("\"");
                }
                if (xOpen != null && xOpen.startsWith("22")) {
                    throw new MysqlDataTruncation(errorBuf.toString(), 0, true, false, 0, 0);
                }
                throw SQLError.createSQLException(errorBuf.toString(), xOpen, errno);
            }
            String serverErrorMessage2 = resultPacket.readString(this.connection.getErrorMessageEncoding());
            clearInputStream();
            if (serverErrorMessage2.indexOf(Messages.getString("MysqlIO.70")) != -1) {
                throw SQLError.createSQLException(new StringBuffer().append(SQLError.get(SQLError.SQL_STATE_COLUMN_NOT_FOUND)).append(", ").append(serverErrorMessage2).toString(), SQLError.SQL_STATE_COLUMN_NOT_FOUND, -1);
            }
            StringBuffer errorBuf2 = new StringBuffer(Messages.getString("MysqlIO.72"));
            errorBuf2.append(serverErrorMessage2);
            errorBuf2.append("\"");
            throw SQLError.createSQLException(new StringBuffer().append(SQLError.get(SQLError.SQL_STATE_GENERAL_ERROR)).append(", ").append(errorBuf2.toString()).toString(), SQLError.SQL_STATE_GENERAL_ERROR, -1);
        }
    }

    private final void sendSplitPackets(Buffer packet) throws SQLException {
        try {
            Buffer headerPacket = this.splitBufRef == null ? null : (Buffer) this.splitBufRef.get();
            if (headerPacket == null) {
                headerPacket = new Buffer(this.maxThreeBytes + 4);
                this.splitBufRef = new SoftReference(headerPacket);
            }
            int len = packet.getPosition();
            int splitSize = this.maxThreeBytes;
            int originalPacketPos = 4;
            byte[] origPacketBytes = packet.getByteBuffer();
            byte[] headerPacketBytes = headerPacket.getByteBuffer();
            while (len >= this.maxThreeBytes) {
                this.packetSequence = (byte) (this.packetSequence + 1);
                headerPacket.setPosition(0);
                headerPacket.writeLongInt(splitSize);
                headerPacket.writeByte(this.packetSequence);
                System.arraycopy(origPacketBytes, originalPacketPos, headerPacketBytes, 4, splitSize);
                int i = splitSize + 4;
                if (!this.useCompression) {
                    this.mysqlOutput.write(headerPacketBytes, 0, splitSize + 4);
                    this.mysqlOutput.flush();
                } else {
                    headerPacket.setPosition(0);
                    Buffer packetToSend = compressPacket(headerPacket, 4, splitSize, 4);
                    int packetLen = packetToSend.getPosition();
                    this.mysqlOutput.write(packetToSend.getByteBuffer(), 0, packetLen);
                    this.mysqlOutput.flush();
                }
                originalPacketPos += splitSize;
                len -= splitSize;
            }
            headerPacket.clear();
            headerPacket.setPosition(0);
            headerPacket.writeLongInt(len - 4);
            this.packetSequence = (byte) (this.packetSequence + 1);
            headerPacket.writeByte(this.packetSequence);
            if (len != 0) {
                System.arraycopy(origPacketBytes, originalPacketPos, headerPacketBytes, 4, len - 4);
            }
            int packetLen2 = len - 4;
            if (!this.useCompression) {
                this.mysqlOutput.write(headerPacket.getByteBuffer(), 0, len);
                this.mysqlOutput.flush();
            } else {
                headerPacket.setPosition(0);
                Buffer packetToSend2 = compressPacket(headerPacket, 4, packetLen2, 4);
                int packetLen3 = packetToSend2.getPosition();
                this.mysqlOutput.write(packetToSend2.getByteBuffer(), 0, packetLen3);
                this.mysqlOutput.flush();
            }
        } catch (IOException ioEx) {
            throw SQLError.createCommunicationsException(this.connection, this.lastPacketSentTimeMs, ioEx);
        }
    }

    private void reclaimLargeSharedSendPacket() {
        if (this.sharedSendPacket != null && this.sharedSendPacket.getCapacity() > MAX_QUERY_SIZE_TO_EXPLAIN) {
            this.sharedSendPacket = new Buffer(MysqlErrorNumbers.ER_ERROR_ON_READ);
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public boolean hadWarnings() {
        return this.hadWarnings;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void scanForAndThrowDataTruncation() throws SQLException {
        if (this.streamingData == null && versionMeetsMinimum(4, 1, 0) && this.connection.getJdbcCompliantTruncation() && this.warningCount > 0) {
            SQLError.convertShowWarningsToSQLWarnings(this.connection, this.warningCount, true);
        }
    }

    private void secureAuth(Buffer packet, int packLength, String user, String password, String database, boolean writeClientParams) throws SQLException {
        if (packet == null) {
            packet = new Buffer(packLength);
        }
        if (writeClientParams) {
            if (this.use41Extensions) {
                if (versionMeetsMinimum(4, 1, 1)) {
                    packet.writeLong(this.clientParam);
                    packet.writeLong(this.maxThreeBytes);
                    packet.writeByte((byte) 8);
                    packet.writeBytesNoNull(new byte[23]);
                } else {
                    packet.writeLong(this.clientParam);
                    packet.writeLong(this.maxThreeBytes);
                }
            } else {
                packet.writeInt((int) this.clientParam);
                packet.writeLongInt(this.maxThreeBytes);
            }
        }
        packet.writeString(user, CODE_PAGE_1252, this.connection);
        if (password.length() != 0) {
            packet.writeString(FALSE_SCRAMBLE, CODE_PAGE_1252, this.connection);
        } else {
            packet.writeString("", CODE_PAGE_1252, this.connection);
        }
        if (this.useConnectWithDb) {
            packet.writeString(database, CODE_PAGE_1252, this.connection);
        }
        send(packet, packet.getPosition());
        if (password.length() > 0) {
            Buffer b = readPacket();
            b.setPosition(0);
            byte[] replyAsBytes = b.getByteBuffer();
            if (replyAsBytes.length == 25 && replyAsBytes[0] != 0) {
                if (replyAsBytes[0] != 42) {
                    try {
                        byte[] buff = Security.passwordHashStage1(password);
                        byte[] passwordHash = new byte[buff.length];
                        System.arraycopy(buff, 0, passwordHash, 0, buff.length);
                        byte[] passwordHash2 = Security.passwordHashStage2(passwordHash, replyAsBytes);
                        byte[] packetDataAfterSalt = new byte[replyAsBytes.length - 5];
                        System.arraycopy(replyAsBytes, 4, packetDataAfterSalt, 0, replyAsBytes.length - 5);
                        byte[] mysqlScrambleBuff = new byte[20];
                        Security.passwordCrypt(packetDataAfterSalt, mysqlScrambleBuff, passwordHash2, 20);
                        Security.passwordCrypt(mysqlScrambleBuff, buff, buff, 20);
                        Buffer packet2 = new Buffer(25);
                        packet2.writeBytesNoNull(buff);
                        this.packetSequence = (byte) (this.packetSequence + 1);
                        send(packet2, 24);
                        return;
                    } catch (NoSuchAlgorithmException e) {
                        throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.91")).append(Messages.getString("MysqlIO.92")).toString(), SQLError.SQL_STATE_GENERAL_ERROR);
                    }
                }
                try {
                    byte[] passwordHash3 = Security.createKeyFromOldPassword(password);
                    byte[] netReadPos4 = new byte[replyAsBytes.length - 5];
                    System.arraycopy(replyAsBytes, 4, netReadPos4, 0, replyAsBytes.length - 5);
                    byte[] mysqlScrambleBuff2 = new byte[20];
                    Security.passwordCrypt(netReadPos4, mysqlScrambleBuff2, passwordHash3, 20);
                    String scrambledPassword = Util.scramble(new String(mysqlScrambleBuff2), password);
                    Buffer packet22 = new Buffer(packLength);
                    packet22.writeString(scrambledPassword, CODE_PAGE_1252, this.connection);
                    this.packetSequence = (byte) (this.packetSequence + 1);
                    send(packet22, 24);
                } catch (NoSuchAlgorithmException e2) {
                    throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.93")).append(Messages.getString("MysqlIO.94")).toString(), SQLError.SQL_STATE_GENERAL_ERROR);
                }
            }
        }
    }

    void secureAuth411(Buffer packet, int packLength, String user, String password, String database, boolean writeClientParams) throws SQLException {
        if (packet == null) {
            packet = new Buffer(packLength);
        }
        if (writeClientParams) {
            if (this.use41Extensions) {
                if (versionMeetsMinimum(4, 1, 1)) {
                    packet.writeLong(this.clientParam);
                    packet.writeLong(this.maxThreeBytes);
                    packet.writeByte((byte) 8);
                    packet.writeBytesNoNull(new byte[23]);
                } else {
                    packet.writeLong(this.clientParam);
                    packet.writeLong(this.maxThreeBytes);
                }
            } else {
                packet.writeInt((int) this.clientParam);
                packet.writeLongInt(this.maxThreeBytes);
            }
        }
        packet.writeString(user);
        if (password.length() != 0) {
            packet.writeByte((byte) 20);
            try {
                packet.writeBytesNoNull(Security.scramble411(password, this.seed));
            } catch (NoSuchAlgorithmException e) {
                throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.95")).append(Messages.getString("MysqlIO.96")).toString(), SQLError.SQL_STATE_GENERAL_ERROR);
            }
        } else {
            packet.writeByte((byte) 0);
        }
        if (this.useConnectWithDb) {
            packet.writeString(database);
        }
        send(packet, packet.getPosition());
        byte savePacketSequence = this.packetSequence;
        this.packetSequence = (byte) (savePacketSequence + 1);
        Buffer reply = checkErrorPacket();
        if (reply.isLastDataPacket()) {
            this.packetSequence = (byte) (savePacketSequence + 1);
            packet.clear();
            String seed323 = this.seed.substring(0, 8);
            packet.writeString(Util.newCrypt(password, seed323));
            send(packet, packet.getPosition());
            checkErrorPacket();
        }
    }

    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r0v3, types: [byte[], byte[][]] */
    private final ResultSetRow unpackBinaryResultSetRow(Field[] fields, Buffer binaryData, int resultSetConcurrency) throws SQLException {
        int numFields = fields.length;
        ?? r0 = new byte[numFields];
        int nullCount = (numFields + 9) / 8;
        byte[] nullBitMask = new byte[nullCount];
        for (int i = 0; i < nullCount; i++) {
            nullBitMask[i] = binaryData.readByte();
        }
        int nullMaskPos = 0;
        int bit = 4;
        for (int i2 = 0; i2 < numFields; i2++) {
            if ((nullBitMask[nullMaskPos] & bit) != 0) {
                r0[i2] = 0;
            } else if (resultSetConcurrency != 1008) {
                extractNativeEncodedColumn(binaryData, fields, i2, r0);
            } else {
                unpackNativeEncodedColumn(binaryData, fields, i2, r0);
            }
            int i3 = bit << 1;
            bit = i3;
            if ((i3 & 255) == 0) {
                bit = 1;
                nullMaskPos++;
            }
        }
        return new ByteArrayRow(r0);
    }

    private final void extractNativeEncodedColumn(Buffer binaryData, Field[] fields, int columnIndex, byte[][] unpackedRowData) throws SQLException {
        Field curField = fields[columnIndex];
        switch (curField.getMysqlType()) {
            case 0:
            case 15:
            case 246:
            case 249:
            case 250:
            case 251:
            case MysqlDefs.FIELD_TYPE_BLOB /* 252 */:
            case 253:
            case 254:
            case 255:
                unpackedRowData[columnIndex] = binaryData.readLenByteArray(0);
                return;
            case 1:
                byte[] bArr = new byte[1];
                bArr[0] = binaryData.readByte();
                unpackedRowData[columnIndex] = bArr;
                return;
            case 2:
            case 13:
                unpackedRowData[columnIndex] = binaryData.getBytes(2);
                return;
            case COMP_HEADER_LENGTH /* 3 */:
            case 9:
                unpackedRowData[columnIndex] = binaryData.getBytes(4);
                return;
            case 4:
                unpackedRowData[columnIndex] = binaryData.getBytes(4);
                return;
            case 5:
                unpackedRowData[columnIndex] = binaryData.getBytes(8);
                return;
            case 6:
                return;
            case 7:
            case 12:
                int length = (int) binaryData.readFieldLength();
                unpackedRowData[columnIndex] = binaryData.getBytes(length);
                return;
            case 8:
                unpackedRowData[columnIndex] = binaryData.getBytes(8);
                return;
            case 10:
                int length2 = (int) binaryData.readFieldLength();
                unpackedRowData[columnIndex] = binaryData.getBytes(length2);
                return;
            case 11:
                int length3 = (int) binaryData.readFieldLength();
                unpackedRowData[columnIndex] = binaryData.getBytes(length3);
                return;
            case SERVER_QUERY_NO_GOOD_INDEX_USED /* 16 */:
                unpackedRowData[columnIndex] = binaryData.readLenByteArray(0);
                return;
            default:
                throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.97")).append(curField.getMysqlType()).append(Messages.getString("MysqlIO.98")).append(columnIndex).append(Messages.getString("MysqlIO.99")).append(fields.length).append(Messages.getString("MysqlIO.100")).toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
    }

    private final void unpackNativeEncodedColumn(Buffer binaryData, Field[] fields, int columnIndex, byte[][] unpackedRowData) throws SQLException {
        Field curField = fields[columnIndex];
        switch (curField.getMysqlType()) {
            case 0:
            case 15:
            case SERVER_QUERY_NO_GOOD_INDEX_USED /* 16 */:
            case 246:
            case 249:
            case 250:
            case 251:
            case MysqlDefs.FIELD_TYPE_BLOB /* 252 */:
            case 253:
            case 254:
                unpackedRowData[columnIndex] = binaryData.readLenByteArray(0);
                return;
            case 1:
                byte tinyVal = binaryData.readByte();
                if (!curField.isUnsigned()) {
                    unpackedRowData[columnIndex] = String.valueOf((int) tinyVal).getBytes();
                    return;
                } else {
                    short unsignedTinyVal = (short) (tinyVal & 255);
                    unpackedRowData[columnIndex] = String.valueOf((int) unsignedTinyVal).getBytes();
                    return;
                }
            case 2:
            case 13:
                short shortVal = (short) binaryData.readInt();
                if (!curField.isUnsigned()) {
                    unpackedRowData[columnIndex] = String.valueOf((int) shortVal).getBytes();
                    return;
                } else {
                    int unsignedShortVal = shortVal & 65535;
                    unpackedRowData[columnIndex] = String.valueOf(unsignedShortVal).getBytes();
                    return;
                }
            case COMP_HEADER_LENGTH /* 3 */:
            case 9:
                int intVal = (int) binaryData.readLong();
                if (!curField.isUnsigned()) {
                    unpackedRowData[columnIndex] = String.valueOf(intVal).getBytes();
                    return;
                } else {
                    unpackedRowData[columnIndex] = String.valueOf(intVal & 4294967295L).getBytes();
                    return;
                }
            case 4:
                float floatVal = Float.intBitsToFloat(binaryData.readIntAsLong());
                unpackedRowData[columnIndex] = String.valueOf(floatVal).getBytes();
                return;
            case 5:
                double doubleVal = Double.longBitsToDouble(binaryData.readLongLong());
                unpackedRowData[columnIndex] = String.valueOf(doubleVal).getBytes();
                return;
            case 6:
                return;
            case 7:
            case 12:
                int length = (int) binaryData.readFieldLength();
                int year = 0;
                int month = 0;
                int day = 0;
                int hour = 0;
                int minute = 0;
                int seconds = 0;
                if (length != 0) {
                    year = binaryData.readInt();
                    month = binaryData.readByte();
                    day = binaryData.readByte();
                    if (length > 4) {
                        hour = binaryData.readByte();
                        minute = binaryData.readByte();
                        seconds = binaryData.readByte();
                    }
                }
                if (year == 0 && month == 0 && day == 0) {
                    if ("convertToNull".equals(this.connection.getZeroDateTimeBehavior())) {
                        unpackedRowData[columnIndex] = null;
                        return;
                    } else {
                        if ("exception".equals(this.connection.getZeroDateTimeBehavior())) {
                            throw SQLError.createSQLException("Value '0000-00-00' can not be represented as java.sql.Timestamp", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
                        }
                        year = 1;
                        month = 1;
                        day = 1;
                    }
                }
                byte[] nanosAsBytes = Integer.toString(0).getBytes();
                int stringLength = 19 + 1 + nanosAsBytes.length;
                byte[] datetimeAsBytes = new byte[stringLength];
                datetimeAsBytes[0] = (byte) Character.forDigit(year / MysqlErrorNumbers.ER_HASHCHK, 10);
                int after1000 = year % MysqlErrorNumbers.ER_HASHCHK;
                datetimeAsBytes[1] = (byte) Character.forDigit(after1000 / 100, 10);
                int after100 = after1000 % 100;
                datetimeAsBytes[2] = (byte) Character.forDigit(after100 / 10, 10);
                datetimeAsBytes[COMP_HEADER_LENGTH] = (byte) Character.forDigit(after100 % 10, 10);
                datetimeAsBytes[4] = 45;
                datetimeAsBytes[5] = (byte) Character.forDigit(month / 10, 10);
                datetimeAsBytes[6] = (byte) Character.forDigit(month % 10, 10);
                datetimeAsBytes[7] = 45;
                datetimeAsBytes[8] = (byte) Character.forDigit(day / 10, 10);
                datetimeAsBytes[9] = (byte) Character.forDigit(day % 10, 10);
                datetimeAsBytes[10] = 32;
                datetimeAsBytes[11] = (byte) Character.forDigit(hour / 10, 10);
                datetimeAsBytes[12] = (byte) Character.forDigit(hour % 10, 10);
                datetimeAsBytes[13] = 58;
                datetimeAsBytes[14] = (byte) Character.forDigit(minute / 10, 10);
                datetimeAsBytes[15] = (byte) Character.forDigit(minute % 10, 10);
                datetimeAsBytes[SERVER_QUERY_NO_GOOD_INDEX_USED] = 58;
                datetimeAsBytes[17] = (byte) Character.forDigit(seconds / 10, 10);
                datetimeAsBytes[18] = (byte) Character.forDigit(seconds % 10, 10);
                datetimeAsBytes[19] = 46;
                for (int j = 0; j < nanosAsBytes.length; j++) {
                    datetimeAsBytes[20 + j] = nanosAsBytes[j];
                }
                unpackedRowData[columnIndex] = datetimeAsBytes;
                return;
            case 8:
                long longVal = binaryData.readLongLong();
                if (!curField.isUnsigned()) {
                    unpackedRowData[columnIndex] = String.valueOf(longVal).getBytes();
                    return;
                } else {
                    BigInteger asBigInteger = ResultSetImpl.convertLongToUlong(longVal);
                    unpackedRowData[columnIndex] = asBigInteger.toString().getBytes();
                    return;
                }
            case 10:
                int year2 = 0;
                int month2 = 0;
                int day2 = 0;
                if (((int) binaryData.readFieldLength()) != 0) {
                    year2 = binaryData.readInt();
                    month2 = binaryData.readByte();
                    day2 = binaryData.readByte();
                }
                if (year2 == 0 && month2 == 0 && day2 == 0) {
                    if ("convertToNull".equals(this.connection.getZeroDateTimeBehavior())) {
                        unpackedRowData[columnIndex] = null;
                        return;
                    } else {
                        if ("exception".equals(this.connection.getZeroDateTimeBehavior())) {
                            throw SQLError.createSQLException("Value '0000-00-00' can not be represented as java.sql.Date", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
                        }
                        year2 = 1;
                        month2 = 1;
                        day2 = 1;
                    }
                }
                byte[] dateAsBytes = new byte[10];
                dateAsBytes[0] = (byte) Character.forDigit(year2 / MysqlErrorNumbers.ER_HASHCHK, 10);
                int after10002 = year2 % MysqlErrorNumbers.ER_HASHCHK;
                dateAsBytes[1] = (byte) Character.forDigit(after10002 / 100, 10);
                int after1002 = after10002 % 100;
                dateAsBytes[2] = (byte) Character.forDigit(after1002 / 10, 10);
                dateAsBytes[COMP_HEADER_LENGTH] = (byte) Character.forDigit(after1002 % 10, 10);
                dateAsBytes[4] = 45;
                dateAsBytes[5] = (byte) Character.forDigit(month2 / 10, 10);
                dateAsBytes[6] = (byte) Character.forDigit(month2 % 10, 10);
                dateAsBytes[7] = 45;
                dateAsBytes[8] = (byte) Character.forDigit(day2 / 10, 10);
                dateAsBytes[9] = (byte) Character.forDigit(day2 % 10, 10);
                unpackedRowData[columnIndex] = dateAsBytes;
                return;
            case 11:
                int length2 = (int) binaryData.readFieldLength();
                int hour2 = 0;
                int minute2 = 0;
                int seconds2 = 0;
                if (length2 != 0) {
                    binaryData.readByte();
                    binaryData.readLong();
                    hour2 = binaryData.readByte();
                    minute2 = binaryData.readByte();
                    seconds2 = binaryData.readByte();
                    if (length2 > 8) {
                        binaryData.readLong();
                    }
                }
                byte[] timeAsBytes = new byte[8];
                timeAsBytes[0] = (byte) Character.forDigit(hour2 / 10, 10);
                timeAsBytes[1] = (byte) Character.forDigit(hour2 % 10, 10);
                timeAsBytes[2] = 58;
                timeAsBytes[COMP_HEADER_LENGTH] = (byte) Character.forDigit(minute2 / 10, 10);
                timeAsBytes[4] = (byte) Character.forDigit(minute2 % 10, 10);
                timeAsBytes[5] = 58;
                timeAsBytes[6] = (byte) Character.forDigit(seconds2 / 10, 10);
                timeAsBytes[7] = (byte) Character.forDigit(seconds2 % 10, 10);
                unpackedRowData[columnIndex] = timeAsBytes;
                return;
            default:
                throw SQLError.createSQLException(new StringBuffer().append(Messages.getString("MysqlIO.97")).append(curField.getMysqlType()).append(Messages.getString("MysqlIO.98")).append(columnIndex).append(Messages.getString("MysqlIO.99")).append(fields.length).append(Messages.getString("MysqlIO.100")).toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
    }

    private void negotiateSSLConnection(String user, String password, String database, int packLength) throws SQLException {
        if (!ExportControlled.enabled()) {
            throw new ConnectionFeatureNotAvailableException(this.connection, this.lastPacketSentTimeMs, null);
        }
        boolean doSecureAuth = false;
        if ((this.serverCapabilities & CLIENT_SECURE_CONNECTION) != 0) {
            this.clientParam |= 32768;
            doSecureAuth = true;
        }
        this.clientParam |= 2048;
        Buffer packet = new Buffer(packLength);
        if (this.use41Extensions) {
            packet.writeLong(this.clientParam);
        } else {
            packet.writeInt((int) this.clientParam);
        }
        send(packet, packet.getPosition());
        ExportControlled.transformSocketToSSLSocket(this);
        packet.clear();
        if (doSecureAuth) {
            if (versionMeetsMinimum(4, 1, 1)) {
                secureAuth411(null, packLength, user, password, database, true);
                return;
            } else {
                secureAuth411(null, packLength, user, password, database, true);
                return;
            }
        }
        if (this.use41Extensions) {
            packet.writeLong(this.clientParam);
            packet.writeLong(this.maxThreeBytes);
        } else {
            packet.writeInt((int) this.clientParam);
            packet.writeLongInt(this.maxThreeBytes);
        }
        packet.writeString(user);
        if (this.protocolVersion > 9) {
            packet.writeString(Util.newCrypt(password, this.seed));
        } else {
            packet.writeString(Util.oldCrypt(password, this.seed));
        }
        if ((this.serverCapabilities & 8) != 0 && database != null && database.length() > 0) {
            packet.writeString(database);
        }
        send(packet, packet.getPosition());
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public int getServerStatus() {
        return this.serverStatus;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public List fetchRowsViaCursor(List fetchedRows, long statementId, Field[] columnTypes, int fetchSize, boolean useBufferRowExplicit) throws SQLException {
        if (fetchedRows == null) {
            fetchedRows = new ArrayList(fetchSize);
        } else {
            fetchedRows.clear();
        }
        this.sharedSendPacket.clear();
        this.sharedSendPacket.writeByte((byte) 28);
        this.sharedSendPacket.writeLong(statementId);
        this.sharedSendPacket.writeLong(fetchSize);
        sendCommand(28, null, this.sharedSendPacket, true, null);
        while (true) {
            ResultSetRow row = nextRow(columnTypes, columnTypes.length, true, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, useBufferRowExplicit, false, null);
            if (row != null) {
                fetchedRows.add(row);
            } else {
                return fetchedRows;
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public long getThreadId() {
        return this.threadId;
    }

    protected boolean useNanosForElapsedTime() {
        return this.useNanosForElapsedTime;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public long getSlowQueryThreshold() {
        return this.slowQueryThreshold;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public String getQueryTimingUnits() {
        return this.queryTimingUnits;
    }
}
